############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
import datetime
# external packages
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.uic
# local import
from mw4.base import transform


class Mount(object):
    """
    the main window class handles the main menu as well as the show and no show part of
    any other window. all necessary processing for functions of that gui will be linked
    to this class. therefore window classes will have a threadpool for managing async
    processing if needed.
    """

    def __init__(self):
        ms = self.app.mount.signals
        ms.locationDone.connect(self.updateLocGUI)
        ms.pointDone.connect(self.updatePointGUI)
        ms.settDone.connect(self.updateSettingGUI)
        ms.settDone.connect(self.updateSetStatGUI)
        ms.settDone.connect(self.updateTrackingGui)

        self.ui.park.clicked.connect(self.changePark)
        self.ui.tracking.clicked.connect(self.changeTracking)
        self.ui.setLunarTracking.clicked.connect(self.setLunarTracking)
        self.ui.setSiderealTracking.clicked.connect(self.setSiderealTracking)
        self.ui.setSolarTracking.clicked.connect(self.setSolarTracking)
        self.ui.stop.clicked.connect(self.stop)
        self.clickable(self.ui.meridianLimitTrack).connect(self.setMeridianLimitTrack)
        self.clickable(self.ui.meridianLimitSlew).connect(self.setMeridianLimitSlew)
        self.clickable(self.ui.horizonLimitHigh).connect(self.setHorizonLimitHigh)
        self.clickable(self.ui.horizonLimitLow).connect(self.setHorizonLimitLow)
        self.clickable(self.ui.slewRate).connect(self.setSlewRate)
        self.clickable(self.ui.siteLatitude).connect(self.setLatitude)
        self.clickable(self.ui.siteLongitude).connect(self.setLongitude)
        self.clickable(self.ui.siteElevation).connect(self.setElevation)

        self.clickable(self.ui.statusUnattendedFlip).connect(self.setUnattendedFlip)
        self.clickable(self.ui.statusDualTracking).connect(self.setDualAxisTracking)
        self.clickable(self.ui.statusRefraction).connect(self.setRefraction)

    def initConfig(self):
        """
        initConfig read the key out of the configuration dict and stores it to the gui
        elements. if some initialisations have to be proceeded with the loaded persistent
        data, they will be launched as well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        self.ui.checkJ2000.setChecked(config.get('checkJ2000', False))
        self.ui.checkJNow.setChecked(config.get('checkJNow', False))
        return True

    def storeConfig(self):
        """
        storeConfig writes the keys to the configuration dict and stores. if some
        saving has to be proceeded to persistent data, they will be launched as
        well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        config['checkJ2000'] = self.ui.checkJ2000.isChecked()
        config['checkJNow'] = self.ui.checkJNow.isChecked()
        return True

    def setupIcons(self):
        """
        setupIcons add icon from standard library to certain buttons for improving the
        gui of the app.

        :return:    True if success for test
        """
        self.wIcon(self.ui.stop, PyQt5.QtWidgets.QStyle.SP_MessageBoxWarning)
        return True

    def updatePointGUI(self, obs):
        """
        updatePointGUI update the gui upon events triggered be the reception of new data
        from the mount. the mount data is polled, so we use this signal as well for the
        update process.

        :param obs:
        :return:    True if ok for testing
        """

        if obs.Alt is not None:
            self.ui.ALT.setText('{0:5.2f}'.format(obs.Alt.degrees))
        else:
            self.ui.ALT.setText('-')

        if obs.Az is not None:
            self.ui.AZ.setText('{0:5.2f}'.format(obs.Az.degrees))
        else:
            self.ui.AZ.setText('-')

        ra = obs.raJNow
        dec = obs.decJNow
        if self.ui.checkJ2000.isChecked():
            if ra is not None and dec is not None and obs.timeJD is not None:
                ra, dec = transform.JNowToJ2000(ra, dec, obs.timeJD)

        if ra is not None:
            raFormat = '{0:02.0f}:{1:02.0f}:{2:02.0f}'
            val = ra.hms()
            raText = raFormat.format(*val)
            self.ui.RA.setText(raText)
            self.ui.RAfloat.setText(f'{ra.hours:3.4f}')
        else:
            self.ui.RA.setText('-')
            self.ui.RAfloat.setText('-')

        if dec is not None:
            decFormat = '{sign}{0:02.0f}:{1:02.0f}:{2:02.0f}'
            val = dec.signed_dms()[1:4]
            decText = decFormat.format(*val, sign='+' if dec.degrees > 0 else '-')
            self.ui.DEC.setText(decText)
            self.ui.DECfloat.setText(f'{dec.degrees:+3.4f}')
        else:
            self.ui.DEC.setText('-')
            self.ui.DECfloat.setText('-')

        if obs.timeJD is not None:
            text = obs.timeJD.utc_strftime('%H:%M:%S')
            self.ui.timeJD.setText(text)
            self.ui.timeUTC.setText('UTC: ' + text)
        else:
            self.ui.timeJD.setText('-')

        if obs.pierside is not None:
            self.ui.pierside.setText('WEST' if obs.pierside == 'W' else 'EAST')
        else:
            self.ui.pierside.setText('-')

        if obs.timeSidereal is not None:
            siderealFormat = '{0:02.0f}:{1:02.0f}:{2:02.0f}'
            val = obs.timeSidereal.hms()
            siderealText = siderealFormat.format(*val)
            self.ui.timeSidereal.setText(siderealText)
        else:
            self.ui.timeSidereal.setText('-')

        if obs.haJNow is not None:
            haFormat = '{0:02.0f}:{1:02.0f}:{2:02.0f}'
            val = obs.haJNow.hms()
            haText = haFormat.format(*val)
            self.ui.HA.setText(haText)
            self.ui.HAfloat.setText(f'{obs.haJNow.hours:3.4f}')
        else:
            self.ui.HA.setText('-')
            self.ui.HAfloat.setText('-')

        return True

    def updateSetStatGUI(self, sett):
        """
        updateSetStatGUI update the gui upon events triggered be the reception of new
        settings from the mount. the mount data is polled, so we use this signal as well
        for the update process.

        :param sett:
        :return:    True if ok for testing
        """

        if sett.UTCExpire is not None:
            ui = self.ui.UTCExpire
            ui.setText(sett.UTCExpire)
            # coloring if close to end:
            now = datetime.datetime.now()
            expire = datetime.datetime.strptime(sett.UTCExpire, '%Y-%m-%d')
            deltaYellow = datetime.timedelta(days=30)
            if now > expire:
                self.changeStyleDynamic(ui, 'color', 'red')
            elif now > expire - deltaYellow:
                self.changeStyleDynamic(ui, 'color', 'yellow')
            else:
                self.changeStyleDynamic(ui, 'color', '')
        else:
            self.ui.UTCExpire.setText('-')

        if sett.statusUnattendedFlip is not None:
            self.ui.statusUnattendedFlip.setText('ON' if sett.statusUnattendedFlip else 'OFF')
        else:
            self.ui.statusUnattendedFlip.setText('-')

        if sett.statusDualTracking is not None:
            self.ui.statusDualTracking.setText('ON' if sett.statusDualTracking else 'OFF')
        else:
            self.ui.statusDualTracking.setText('-')

        if sett.statusRefraction is not None:
            self.ui.statusRefraction.setText('ON' if sett.statusRefraction else 'OFF')
        else:
            self.ui.statusRefraction.setText('-')

        if sett.gpsSynced is not None:
            self.ui.statusGPSSynced.setText('YES' if sett.gpsSynced else 'NO')
        else:
            self.ui.statusGPSSynced.setText('-')

        if sett.wakeOnLan is not None:
            self.ui.statusWOL.setText(sett.wakeOnLan)
        else:
            self.ui.statusWOL.setText('-')

        return True

    def updateSettingGUI(self, sett):
        """
        updateSettingGUI update the gui upon events triggered be the reception of new
        settings from the mount. the mount data is polled, so we use this signal as well
        for the update process.

        :return:    True if ok for testing
        """

        if sett.slewRate is not None:
            self.ui.slewRate.setText('{0:2.0f}'.format(sett.slewRate))
        else:
            self.ui.slewRate.setText('-')

        if sett.timeToFlip is not None:
            self.ui.timeToFlip.setText('{0:3.0f}'.format(sett.timeToFlip))
        else:
            self.ui.timeToFlip.setText('-')

        if sett.timeToMeridian() is not None:
            self.ui.timeToMeridian.setText('{0:3.0f}'.format(sett.timeToMeridian()))
        else:
            self.ui.timeToMeridian.setText('-')

        if sett.refractionTemp is not None:
            self.ui.refractionTemp.setText('{0:+4.1f}'.format(sett.refractionTemp))
            self.ui.refractionTemp1.setText('{0:+4.1f}'.format(sett.refractionTemp))
        else:
            self.ui.refractionTemp.setText('-')
            self.ui.refractionTemp1.setText('-')

        if sett.refractionPress is not None:
            self.ui.refractionPress.setText('{0:6.1f}'.format(sett.refractionPress))
            self.ui.refractionPress1.setText('{0:6.1f}'.format(sett.refractionPress))
        else:
            self.ui.refractionPress.setText('-')
            self.ui.refractionPress1.setText('-')

        if sett.meridianLimitTrack is not None:
            self.ui.meridianLimitTrack.setText(str(sett.meridianLimitTrack))
        else:
            self.ui.meridianLimitTrack.setText('-')

        if sett.meridianLimitSlew is not None:
            self.ui.meridianLimitSlew.setText(str(sett.meridianLimitSlew))
        else:
            self.ui.meridianLimitSlew.setText('-')

        if sett.horizonLimitLow is not None:
            self.ui.horizonLimitLow.setText(str(sett.horizonLimitLow))
        else:
            self.ui.horizonLimitLow.setText('-')

        if sett.horizonLimitHigh is not None:
            self.ui.horizonLimitHigh.setText(str(sett.horizonLimitHigh))
        else:
            self.ui.horizonLimitHigh.setText('-')

        return True

    def updateLocGUI(self, location):
        """
        updateLocGUI update the gui upon events triggered be the reception of new
        settings from the mount. the mount data is polled, so we use this signal as well
        for the update process.

        :return:    True if ok for testing
        """

        if location is not None:
            lon = location.longitude.dstr().replace('deg', '')
            self.ui.siteLongitude.setText(lon)
            lat = location.latitude.dstr().replace('deg', '')
            self.ui.siteLatitude.setText(lat)
            self.ui.siteElevation.setText(str(location.elevation.m))

        return True

    def updateTrackingGui(self, sett):
        """
        updateTrackingGui update the gui upon events triggered be the reception of new
        settings from the mount. the mount data is polled, so we use this signal as well
        for the update process.

        :param sett:
        :return:    True if ok for testing
        """

        if sett.checkRateLunar():
            self.changeStyleDynamic(self.ui.setLunarTracking, 'running', 'true')
            self.changeStyleDynamic(self.ui.setSiderealTracking, 'running', 'false')
            self.changeStyleDynamic(self.ui.setSolarTracking, 'running', 'false')
        elif sett.checkRateSidereal():
            self.changeStyleDynamic(self.ui.setLunarTracking, 'running', 'false')
            self.changeStyleDynamic(self.ui.setSiderealTracking, 'running', 'true')
            self.changeStyleDynamic(self.ui.setSolarTracking, 'running', 'false')
        elif sett.checkRateSolar():
            self.changeStyleDynamic(self.ui.setLunarTracking, 'running', 'false')
            self.changeStyleDynamic(self.ui.setSiderealTracking, 'running', 'false')
            self.changeStyleDynamic(self.ui.setSolarTracking, 'running', 'true')

        return True

    def changeTracking(self):
        """

        :return:
        """

        obs = self.app.mount.obsSite
        if obs.status == 0:
            suc = obs.stopTracking()
            if not suc:
                self.app.message.emit('Cannot stop tracking', 2)
            else:
                self.app.message.emit('Stopped tracking', 0)
        else:
            suc = obs.startTracking()
            if not suc:
                self.app.message.emit('Cannot start tracking', 2)
            else:
                self.app.message.emit('Started tracking', 0)

        return True

    def changePark(self):
        """

        :return:
        """

        obs = self.app.mount.obsSite
        if obs.status == 5:
            suc = obs.unpark()
            if not suc:
                self.app.message.emit('Cannot unpark mount', 2)
            else:
                self.app.message.emit('Mount unparked', 0)
        else:
            suc = obs.park()
            if not suc:
                self.app.message.emit('Cannot park mount', 2)
            else:
                self.app.message.emit('Mount parked', 0)

        return True

    def setLunarTracking(self):
        """

        :return:
        """

        obs = self.app.mount.obsSite
        suc = obs.setLunarTracking()
        if not suc:
            self.app.message.emit('Cannot set tracking to Lunar', 2)
            return False
        else:
            self.app.message.emit('Tracking set to Lunar', 0)
            return True

    def setSiderealTracking(self):
        """

        :return:
        """

        obs = self.app.mount.obsSite
        suc = obs.setSiderealTracking()
        if not suc:
            self.app.message.emit('Cannot set tracking to Sidereal', 2)
            return False
        else:
            self.app.message.emit('Tracking set to Sidereal', 0)
            return True

    def setSolarTracking(self):
        """

        :return:
        """

        obs = self.app.mount.obsSite
        suc = obs.setSolarTracking()
        if not suc:
            self.app.message.emit('Cannot set tracking to Solar', 2)
            return False
        else:
            self.app.message.emit('Tracking set to Solar', 0)
            return True

    def stop(self):
        """

        :return:
        """

        obs = self.app.mount.obsSite
        suc = obs.stop()
        if not suc:
            self.app.message.emit('Cannot stop mount', 2)
            return False
        else:
            self.app.message.emit('Mount stopped', 0)
            return True

    def setMeridianLimitTrack(self):
        """
        setMeridianLimitTrack implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        sett = self.app.mount.sett
        msg = PyQt5.QtWidgets.QMessageBox
        obs = self.app.mount.obsSite
        actValue = sett.meridianLimitTrack
        if actValue is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getInt(self,
                               'Set Meridian Limit Track',
                               'Value (-20-20):',
                               actValue,
                               -20,
                               20,
                               1,
                               )

        if not ok:
            return False

        if obs.setMeridianLimitTrack(value):
            self.app.message.emit(f'Meridian Limit Track: [{value}]', 0)
            return True
        else:
            self.app.message.emit('Meridian Limit Track cannot be set', 2)
            return False

    def setMeridianLimitSlew(self):
        """
        setMeridianLimitSlew implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        sett = self.app.mount.sett
        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        actValue = sett.meridianLimitSlew
        if actValue is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getInt(self,
                               'Set Meridian Limit Slew',
                               'Value (-20-20):',
                               actValue,
                               -20,
                               20,
                               1,
                               )

        if not ok:
            return False

        if obs.setMeridianLimitSlew(value):
            self.app.message.emit(f'Meridian Limit Slew: [{value}]', 0)
            return True
        else:
            self.app.message.emit('Meridian Limit Slew cannot be set', 2)
            return False

    def setHorizonLimitHigh(self):
        """
        setHorizonLimitHigh implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        sett = self.app.mount.sett
        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        actValue = sett.horizonLimitHigh
        if actValue is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getInt(self,
                               'Set Horizon Limit High',
                               'Value (0-90):',
                               actValue,
                               0,
                               90,
                               1,
                               )

        if not ok:
            return False

        if obs.setHorizonLimitHigh(value):
            self.app.message.emit(f'Horizon Limit High: [{value}]', 0)
            return True
        else:
            self.app.message.emit('Horizon Limit High cannot be set', 2)
            return False

    def setHorizonLimitLow(self):
        """
        setHorizonLimitLow implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        sett = self.app.mount.sett
        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        actValue = sett.horizonLimitLow
        if actValue is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False

        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getInt(self,
                               'Set Horizon Limit Low',
                               'Value (0-90):',
                               actValue,
                               0,
                               90,
                               1,
                               )

        if not ok:
            return False

        if obs.setHorizonLimitLow(value):
            self.app.message.emit(f'Horizon Limit Low: [{value}]', 0)
            return True
        else:
            self.app.message.emit('Horizon Limit Low cannot be set', 2)
            return False

    def setSlewRate(self):
        """
        setSlewRate implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """
        sett = self.app.mount.sett
        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        actValue = sett.slewRate
        if actValue is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False

        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getInt(self,
                               'Set Slew Rate',
                               'Value (1-20):',
                               actValue,
                               1,
                               20,
                               1,
                               )

        if not ok:
            return False

        if obs.setSlewRate(value):
            self.app.message.emit(f'Slew Rate: [{value}]', 0)
            return True
        else:
            self.app.message.emit('Slew Rate cannot be set', 2)
            return False

    def setLongitude(self):
        """
        setSiteLongitude implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        if obs.location is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getText(self,
                                'Set Site Longitude',
                                'Value: (East positive)',
                                PyQt5.QtWidgets.QLineEdit.Normal,
                                obs.location.longitude.dstr(),
                                )
        if not ok:
            return False

        if obs.setLongitude(value):
            self.app.message.emit(f'Longitude: [{value}]', 0)
            self.app.mount.getLocation()
            return True
        else:
            self.app.message.emit('Longitude cannot be set', 2)
            return False

    def setLatitude(self):
        """
        setSiteLatitude implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        if obs.location is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getText(self,
                                'Set Site Latitude',
                                'Value:',
                                PyQt5.QtWidgets.QLineEdit.Normal,
                                obs.location.latitude.dstr(),
                                )
        if not ok:
            return False

        if obs.setLatitude(value):
            self.app.message.emit(f'Latitude: [{value}]', 0)
            self.app.mount.getLocation()
            return True
        else:
            self.app.message.emit('Latitude cannot be set', 2)
            return False

    def setElevation(self):
        """
        setSiteElevation implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        if obs.location is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getDouble(self,
                                  'Set Site Elevation',
                                  'Value: (meters)',
                                  obs.location.elevation.m,
                                  0,
                                  8000,
                                  1,
                                  )
        if not ok:
            return False

        if obs.setElevation(value):
            self.app.message.emit(f'Elevation: [{value}]', 0)
            self.app.mount.getLocation()
            return True
        else:
            self.app.message.emit('Elevation cannot be set', 2)

    def setUnattendedFlip(self):
        """
        setUnattendedFlip implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        sett = self.app.mount.sett
        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        if sett.statusUnattendedFlip is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getItem(self,
                                'Set Unattended Flip',
                                'Value: On / Off',
                                ['ON', 'OFF'],
                                0,
                                False,
                                )
        if not ok:
            return False

        suc = obs.setUnattendedFlip(value == 'ON')
        if suc:
            self.app.message.emit(f'Unattended flip set to [{value}]', 0)
        else:
            self.app.message.emit('Unattended flip cannot be set', 2)

    def setDualAxisTracking(self):
        """
        setDualAxisTracking implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        sett = self.app.mount.sett
        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        if sett.statusDualTracking is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getItem(self,
                                'Set Dual Axis Tracking',
                                'Value: On / Off',
                                ['ON', 'OFF'],
                                0,
                                False,
                                )
        if not ok:
            return False

        suc = obs.setDualAxisTracking(value == 'ON')
        if suc:
            self.app.message.emit(f'Dual axis tracking set to [{value}]', 0)
        else:
            self.app.message.emit('Dual axis tracking cannot be set', 2)

    def setRefraction(self):
        """
        setRefractionCorrection implements a modal dialog for entering the value

        :return:    success as bool if value could be changed
        """

        sett = self.app.mount.sett
        obs = self.app.mount.obsSite
        msg = PyQt5.QtWidgets.QMessageBox
        if sett.statusRefraction is None:
            msg.critical(self,
                         'Error Message',
                         'Value cannot be set when mount not connected !')
            return False
        dlg = PyQt5.QtWidgets.QInputDialog()
        value, ok = dlg.getItem(self,
                                'Set Refraction Correction',
                                'Value: On / Off',
                                ['ON', 'OFF'],
                                0,
                                False,
                                )
        if not ok:
            return False

        suc = obs.setRefraction(value == 'ON')
        if suc:
            self.app.message.emit(f'Refraction correction set to [{value}]', 0)
        else:
            self.app.message.emit('Refraction correction cannot be set', 2)



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
from datetime import datetime
# external packages
import PyQt5
import numpy as np
# local imports
from mw4.base import indiClass


class DomeSignals(PyQt5.QtCore.QObject):
    """
    The DomeSignals class offers a list of signals to be used and instantiated by
    the Mount class to get signals for triggers for finished tasks to
    enable a gui to update their values transferred to the caller back.

    This has to be done in a separate class as the signals have to be subclassed from
    QObject and the Mount class itself is subclassed from object
    """

    __all__ = ['DomeSignals']
    version = '1.0.0'

    azimuth = PyQt5.QtCore.pyqtSignal(object)
    slewFinished = PyQt5.QtCore.pyqtSignal()
    message = PyQt5.QtCore.pyqtSignal(object)


class Dome(indiClass.IndiClass):
    """
    the class Dome inherits all information and handling of the Dome device. there will be
    some parameters who will define the slewing position of the dome relating to the mount.

        >>> fw = Dome(
        >>>           app=app
        >>>           host=host
        >>>           name=''
        >>>          )
    """

    __all__ = ['Dome',
               ]

    version = '0.100.0'
    logger = logging.getLogger(__name__)

    # update rate to 1000 milli seconds for setting indi server
    UPDATE_RATE = 1000

    def __init__(self,
                 app=None,
                 host=None,
                 name='',
                 ):
        super().__init__(host=host,
                         name=name
                         )

        self.app = app
        self.signals = DomeSignals()
        self._settlingTime = 0

        self.azimuth = -1
        self.slewing = False

        self.app.update3s.connect(self.updateStatus)
        self.settlingWait = PyQt5.QtCore.QTimer()
        self.settlingWait.setSingleShot(True)
        self.settlingWait.timeout.connect(self.waitSettlingAndEmit)

    @property
    def settlingTime(self):
        return self._settlingTime * 1000

    @settlingTime.setter
    def settlingTime(self, value):
        self._settlingTime = value

    def setUpdateConfig(self, deviceName):
        """
        _setUpdateRate corrects the update rate of dome devices to get an defined
        setting regardless, what is setup in server side.

        :param deviceName:
        :return: success
        """

        if deviceName != self.name:
            return False

        if self.device is None:
            return False

        # setting polling updates in driver
        update = self.device.getNumber('POLLING_PERIOD')

        if 'PERIOD_MS' not in update:
            return False

        if update.get('PERIOD_MS', 0) == self.UPDATE_RATE:
            return True

        update['PERIOD_MS'] = self.UPDATE_RATE
        suc = self.client.sendNewNumber(deviceName=deviceName,
                                        propertyName='POLLING_PERIOD',
                                        elements=update,
                                        )

        return suc

    def updateStatus(self):
        """
        updateStatus emits the actual azimuth status every 3 second in case of opening a
        window and get the signals late connected as INDI does nt repeat any signal of it's
        own

        :return: true for test purpose
        """

        self.signals.azimuth.emit(self.azimuth)

        return True

    def waitSettlingAndEmit(self):
        """
        waitSettlingAndEmit emit the signal for slew finished

        :return: true for test purpose
        """

        self.signals.message.emit('')
        self.signals.slewFinished.emit()

        return True

    def updateNumber(self, deviceName, propertyName):
        """
        updateNumber is called whenever a new number is received in client. it runs
        through the device list and writes the number data to the according locations.

        :param deviceName:
        :param propertyName:
        :return:
        """

        if self.device is None:
            return False
        if deviceName != self.name:
            return False

        for element, value in self.device.getNumber(propertyName).items():
            key = propertyName + '.' + element
            self.data[key] = value
            # print(propertyName, element, value)

            if element != 'DOME_ABSOLUTE_POSITION':
                continue

            # starting condition: don't do anything
            if self.azimuth == -1:
                self.azimuth = value
                continue

            # send trigger for new data
            self.signals.azimuth.emit(self.azimuth)

            # calculate the stop slewing condition
            isSlewing = (self.device.ABS_DOME_POSITION['state'] == 'Busy')
            if isSlewing:
                self.signals.message.emit('slewing')
            if self.slewing and not isSlewing:
                # start timer for settling time and emit signal afterwards
                self.settlingWait.start(self.settlingTime)

            # store for the next cycle
            self.azimuth = value
            self.slewing = isSlewing

        return True

    def calcAzDomeFromMount(self,
                            azimuthMount=0,
                            domeRadius=0, shutterWidth=0,
                            offsetN=0, offsetE=0,
                            offsetUp=0, offsetOTA=0,
                            pierside='E'):
        """

        :param azimuthMount: azimuth of the telescope

        :param domeRadius: radius at Equator - The radius of your dome at the equator.  This
        should be measured from where your shutter opening is.  For example if your dome is
        skinned on the outside you would measure from the outside diameter.  If it is
        skinned on the inside you would measure the inside radius.

        :param shutterWidth: width of the shutter

        :param offsetN: North Offset - The offset from the center of intersection of the
        Right Ascension and Declination axis to the center of the dome.  If the RA/Dec
        intersection is north of the dome center this value is positive.  If the RA/Dec
        intersection is south of this location the value should be negative.

        :param offsetE: East Offset - The offset from the center of the intersection of the
        Right Ascension and Declination axis to the center of the dome.  If the RA/Dec
        intersection is east of the dome center this value is positive.  If the RA/Dec
        intersection is west of this location the value should be negative.

        :param offsetUp: Vertical Offset - The offset from the center of the intersection
        of the Right Ascension and Declination axis to the center of the dome.If the RA/Dec
        intersection is above the dome center this value is positive.  If the RA/Dec
        intersection is below this location the value should be negative.

        :param offsetOTA: The distance from the center of the Right Ascension axis to the
        center of the telescope.  This value can vary depending on how your scopes are
        setup on your mount.  It is best to use some trial and error here.  If the top of
        your scope is being eclipsed by your dome increase this value.  If the bottom of
        your scope is being eclipsed decrease this value.

        :param pierside: side of the pier, where the OTA is intended to be after slewing.

        :return: azimuthDome calculated for right position for dome
        """
        azimuthDome = azimuthMount

        return azimuthDome

    def slewToAltAz(self, altitudeMount=0, azimuthMount=0):
        """
        slewToAltAz sends a command to the dome to move to azimuth / altitude. if a dome
        does support this

        :param altitudeMount:
        :param azimuthMount:
        :return: success
        """

        if self.device is None:
            return False

        if self.name is None or not self.name:
            return False

        position = self.device.getNumber('ABS_DOME_POSITION')

        if 'DOME_ABSOLUTE_POSITION' not in position:
            return False

        azimuthDome = self.calcAzDomeFromMount(azimuthMount=azimuthMount)

        position['DOME_ABSOLUTE_POSITION'] = azimuthDome

        suc = self.client.sendNewNumber(deviceName=self.name,
                                        propertyName='ABS_DOME_POSITION',
                                        elements=position,
                                        )

        if suc:
            self.slewing = True

        return suc



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
# external packages
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.uic
# local import


class AlignMount(object):
    """
    the main window class handles the main menu as well as the show and no show part of
    any other window. all necessary processing for functions of that gui will be linked
    to this class. therefore window classes will have a threadpool for managing async
    processing if needed.
    """

    def __init__(self):
        self.lastModelType = ''

        ms = self.app.mount.signals
        ms.alignDone.connect(self.updateAlignGUI)
        ms.alignDone.connect(self.updateTurnKnobsGUI)

        self.ui.genAlignBuild.clicked.connect(self.genAlignBuild)
        self.ui.genAlignBuildFile.clicked.connect(self.genAlignBuildFile)
        self.ui.altBase.valueChanged.connect(self.genAlignBuild)
        self.ui.azBase.valueChanged.connect(self.genAlignBuild)
        self.ui.numberBase.valueChanged.connect(self.genAlignBuild)
        self.ui.saveAlignBuildPoints.clicked.connect(self.saveAlignBuildFile)
        self.ui.saveAlignBuildPointsAs.clicked.connect(self.saveAlignBuildFileAs)
        self.ui.loadAlignBuildPoints.clicked.connect(self.loadAlignBuildFile)

    def initConfig(self):
        """
        initConfig read the key out of the configuration dict and stores it to the gui
        elements. if some initialisations have to be proceeded with the loaded persistent
        data, they will be launched as well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        self.ui.alignBuildPFileName.setText(config.get('alignBuildPFileName', ''))

        self.ui.altBase.valueChanged.disconnect(self.genAlignBuild)
        self.ui.azBase.valueChanged.disconnect(self.genAlignBuild)
        self.ui.numberBase.valueChanged.disconnect(self.genAlignBuild)

        self.ui.altBase.setValue(config.get('altBase', 30))
        self.ui.azBase.setValue(config.get('azBase', 45))
        self.ui.numberBase.setValue(config.get('numberBase', 3))

        # initialising the signal slot connections after the value are set, because
        # otherwise we get a first value changed signal just when populating
        # the initial data. this should not happen.
        self.ui.altBase.valueChanged.connect(self.genAlignBuild)
        self.ui.azBase.valueChanged.connect(self.genAlignBuild)
        self.ui.numberBase.valueChanged.connect(self.genAlignBuild)

        return True

    def storeConfig(self):
        """
        storeConfig writes the keys to the configuration dict and stores. if some
        saving has to be proceeded to persistent data, they will be launched as
        well in this method.

        :return: True for test purpose
        """
        config = self.app.config['mainW']
        config['alignBuildPFileName'] = self.ui.alignBuildPFileName.text()
        config['altBase'] = self.ui.altBase.value()
        config['azBase'] = self.ui.azBase.value()
        config['numberBase'] = self.ui.numberBase.value()
        return True

    def setupIcons(self):
        """
        setupIcons add icon from standard library to certain buttons for improving the
        gui of the app.

        :return:    True if success for test
        """

        self.wIcon(self.ui.genAlignBuild, PyQt5.QtWidgets.QStyle.SP_DialogApplyButton)
        self.wIcon(self.ui.plateSolveSync, PyQt5.QtWidgets.QStyle.SP_DialogApplyButton)
        pixmap = PyQt5.QtGui.QPixmap(':/azimuth1.png')
        self.ui.picAZ.setPixmap(pixmap)
        pixmap = PyQt5.QtGui.QPixmap(':/altitude1.png')
        self.ui.picALT.setPixmap(pixmap)
        return True

    def updateAlignGUI(self, model):
        """
        updateAlignGUI shows the data which is received through the getain command. this is
        mainly polar and ortho errors as well as basic model data.

        :param model:
        :return:    True if ok for testing
        """

        if model.numberStars is not None:
            text = str(model.numberStars)
        else:
            text = '-'
        self.ui.numberStars.setText(text)
        self.ui.numberStars1.setText(text)

        if model.terms is not None:
            text = str(model.terms)
        else:
            text = '-'
        self.ui.terms.setText(text)

        if model.errorRMS is not None:
            text = str(model.errorRMS)
        else:
            text = '-'
        self.ui.errorRMS.setText(text)
        self.ui.errorRMS1.setText(text)

        if model.positionAngle is not None:
            text = f'{model.positionAngle.degrees:5.1f}'
        else:
            text = '-'
        self.ui.positionAngle.setText(text)

        if model.polarError is not None:
            text = f'{model.polarError.degrees * 60:5.2f}'
        else:
            text = '-'
        self.ui.polarError.setText(text)

        if model.orthoError is not None:
            text = f'{model.orthoError.degrees * 60:5.2f}'
        else:
            text = '-'
        self.ui.orthoError.setText(text)

        if model.azimuthError is not None:
            text = f'{model.azimuthError.degrees:5.1f}'
        else:
            text = '-'
        self.ui.azimuthError.setText(text)

        if model.altitudeError is not None:
            text = f'{model.altitudeError.degrees:5.1f}'
        else:
            text = '-'
        self.ui.altitudeError.setText(text)

        return True

    def updateTurnKnobsGUI(self, model):
        """
        updateTurnKnobsGUI shows the data which is received through the getain command.
        this is mainly polar and ortho errors as well as basic model data.

        :param model:
        :return:    True if ok for testing
        """

        if model.azimuthTurns is not None:
            if model.azimuthTurns > 0:
                text = '{0:3.1f} revs left'.format(abs(model.azimuthTurns))
            else:
                text = '{0:3.1f} revs right'.format(abs(model.azimuthTurns))
        else:
            text = '-'
        self.ui.azimuthTurns.setText(text)

        if model.altitudeTurns is not None:
            if model.altitudeTurns > 0:
                text = '{0:3.1f} revs down'.format(abs(model.altitudeTurns))
            else:
                text = '{0:3.1f} revs up'.format(abs(model.altitudeTurns))
        else:
            text = '-'
        self.ui.altitudeTurns.setText(text)

        return True

    def genAlignBuild(self):
        """
        genAlignBuild generates a grid of point for model build based on gui data. the cols
        have to be on even numbers.

        :return: success
        """

        self.lastModelType = 'align-grid'
        altBase = self.ui.altBase.value()
        azBase = self.ui.azBase.value()
        numberBase = self.ui.numberBase.value()
        suc = self.app.data.genAlign(altBase=altBase,
                                     azBase=azBase,
                                     numberBase=numberBase,
                                     )
        if not suc:
            return False
        self.autoDeletePoints()
        return True

    def loadAlignBuildFile(self):
        """
        loadAlignBuildFile calls a file selector box and selects the filename to be loaded

        :return: success
        """

        folder = self.app.mwGlob['configDir']
        loadFilePath, fileName, ext = self.openFile(self,
                                                    'Open align build point file',
                                                    folder,
                                                    'Build point files (*.bpts)',
                                                    )
        if not loadFilePath:
            return False
        suc = self.app.data.loadBuildP(fileName=fileName)
        if suc:
            self.ui.alignBuildPFileName.setText(fileName)
            self.app.message.emit('Align build file [{0}] loaded'.format(fileName), 0)
        else:
            self.app.message.emit('Align build file [{0}] cannot no be loaded'
                                  .format(fileName), 2)
        return True

    def saveAlignBuildFile(self):
        """
        saveAlignBuildFile calls saving the build file

        :return: success
        """

        fileName = self.ui.alignBuildPFileName.text()
        if not fileName:
            self.app.message.emit('Align build points file name not given', 2)
            return False
        suc = self.app.data.saveBuildP(fileName=fileName)
        if suc:
            self.app.message.emit('Align build file [{0}] saved'.format(fileName), 0)
        else:
            self.app.message.emit('Align build file [{0}] cannot no be saved'
                                  .format(fileName), 2)
        return True

    def saveAlignBuildFileAs(self):
        """
        saveAlignBuildFileAs calls a file selector box and selects the filename to be save

        :return: success
        """

        folder = self.app.mwGlob['configDir']
        saveFilePath, fileName, ext = self.saveFile(self,
                                                    'Save align build point file',
                                                    folder,
                                                    'Build point files (*.bpts)',
                                                    )
        if not saveFilePath:
            return False
        suc = self.app.data.saveBuildP(fileName=fileName)
        if suc:
            self.ui.alignBuildPFileName.setText(fileName)
            self.app.message.emit('Align build file [{0}] saved'.format(fileName), 0)
        else:
            self.app.message.emit('Align build file [{0}] cannot no be saved'
                                  .format(fileName), 2)
        return True

    def genAlignBuildFile(self):
        """
        genAlignBuildFile tries to load a give build point file and displays it for usage.

        :return: success
        """

        fileName = self.ui.alignBuildPFileName.text()
        if not fileName:
            self.app.message.emit('Align build points file name not given', 2)
            return False

        self.lastModelType = 'align-file'
        suc = self.app.data.loadBuildP(fileName=fileName)
        if not suc:
            text = 'Align build points file [{0}] could not be loaded'.format(fileName)
            self.app.message.emit(text, 2)
            return False
        self.autoDeletePoints()
        return True



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../mw4/gui/widgets/image.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ImageDialog(object):
    def setupUi(self, ImageDialog):
        ImageDialog.setObjectName("ImageDialog")
        ImageDialog.resize(800, 600)
        ImageDialog.setMinimumSize(QtCore.QSize(800, 600))
        ImageDialog.setMaximumSize(QtCore.QSize(1600, 1200))
        ImageDialog.setSizeIncrement(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        ImageDialog.setFont(font)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(ImageDialog)
        self.verticalLayout_2.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout_2.setSpacing(4)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setContentsMargins(4, 4, 4, 4)
        self.horizontalLayout_5.setSpacing(4)
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.exposesettings = QtWidgets.QGridLayout()
        self.exposesettings.setContentsMargins(4, 4, 4, 4)
        self.exposesettings.setHorizontalSpacing(8)
        self.exposesettings.setVerticalSpacing(4)
        self.exposesettings.setObjectName("exposesettings")
        self.exposeN = QtWidgets.QPushButton(ImageDialog)
        self.exposeN.setMinimumSize(QtCore.QSize(80, 25))
        self.exposeN.setObjectName("exposeN")
        self.exposesettings.addWidget(self.exposeN, 1, 0, 1, 1)
        self.solve = QtWidgets.QPushButton(ImageDialog)
        self.solve.setMinimumSize(QtCore.QSize(80, 25))
        self.solve.setObjectName("solve")
        self.exposesettings.addWidget(self.solve, 2, 0, 1, 1)
        self.abortSolve = QtWidgets.QPushButton(ImageDialog)
        self.abortSolve.setEnabled(False)
        self.abortSolve.setMinimumSize(QtCore.QSize(80, 25))
        self.abortSolve.setObjectName("abortSolve")
        self.exposesettings.addWidget(self.abortSolve, 2, 1, 1, 1)
        self.abortImage = QtWidgets.QPushButton(ImageDialog)
        self.abortImage.setEnabled(False)
        self.abortImage.setMinimumSize(QtCore.QSize(80, 25))
        self.abortImage.setObjectName("abortImage")
        self.exposesettings.addWidget(self.abortImage, 1, 1, 1, 1)
        self.checkAutoSolve = QtWidgets.QCheckBox(ImageDialog)
        self.checkAutoSolve.setMinimumSize(QtCore.QSize(0, 25))
        self.checkAutoSolve.setObjectName("checkAutoSolve")
        self.exposesettings.addWidget(self.checkAutoSolve, 2, 5, 1, 1)
        self.load = QtWidgets.QPushButton(ImageDialog)
        self.load.setMinimumSize(QtCore.QSize(80, 25))
        self.load.setObjectName("load")
        self.exposesettings.addWidget(self.load, 0, 5, 1, 1)
        self.imageFileName = QtWidgets.QLineEdit(ImageDialog)
        self.imageFileName.setEnabled(False)
        self.imageFileName.setMinimumSize(QtCore.QSize(200, 25))
        self.imageFileName.setObjectName("imageFileName")
        self.exposesettings.addWidget(self.imageFileName, 0, 1, 1, 4)
        self.expose = QtWidgets.QPushButton(ImageDialog)
        self.expose.setMinimumSize(QtCore.QSize(80, 25))
        self.expose.setObjectName("expose")
        self.exposesettings.addWidget(self.expose, 0, 0, 1, 1)
        self.checkStackImages = QtWidgets.QCheckBox(ImageDialog)
        self.checkStackImages.setEnabled(True)
        self.checkStackImages.setObjectName("checkStackImages")
        self.exposesettings.addWidget(self.checkStackImages, 1, 5, 1, 1)
        self.horizontalLayout_5.addLayout(self.exposesettings)
        self.line_2 = QtWidgets.QFrame(ImageDialog)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout_5.addWidget(self.line_2)
        self.groupBox_3 = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox_3.setObjectName("groupBox_3")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.groupBox_3)
        self.verticalLayout_5.setContentsMargins(10, 4, 10, 4)
        self.verticalLayout_5.setSpacing(4)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.checkUseWCS = QtWidgets.QCheckBox(self.groupBox_3)
        self.checkUseWCS.setMinimumSize(QtCore.QSize(80, 25))
        self.checkUseWCS.setObjectName("checkUseWCS")
        self.verticalLayout_5.addWidget(self.checkUseWCS)
        self.line_5 = QtWidgets.QFrame(self.groupBox_3)
        self.line_5.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_5.setMidLineWidth(1)
        self.line_5.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_5.setObjectName("line_5")
        self.verticalLayout_5.addWidget(self.line_5)
        self.checkShowGrid = QtWidgets.QCheckBox(self.groupBox_3)
        self.checkShowGrid.setObjectName("checkShowGrid")
        self.verticalLayout_5.addWidget(self.checkShowGrid)
        self.checkShowCrosshair = QtWidgets.QCheckBox(self.groupBox_3)
        self.checkShowCrosshair.setObjectName("checkShowCrosshair")
        self.verticalLayout_5.addWidget(self.checkShowCrosshair)
        self.horizontalLayout_5.addWidget(self.groupBox_3)
        self.line_4 = QtWidgets.QFrame(ImageDialog)
        self.line_4.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_4.setMidLineWidth(1)
        self.line_4.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_4.setObjectName("line_4")
        self.horizontalLayout_5.addWidget(self.line_4)
        self.attributsgroup = QtWidgets.QGroupBox(ImageDialog)
        self.attributsgroup.setObjectName("attributsgroup")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.attributsgroup)
        self.verticalLayout_6.setContentsMargins(10, 4, 15, 4)
        self.verticalLayout_6.setSpacing(4)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem)
        self.hasWCS = QtWidgets.QLabel(self.attributsgroup)
        self.hasWCS.setMinimumSize(QtCore.QSize(0, 16))
        self.hasWCS.setObjectName("hasWCS")
        self.verticalLayout_6.addWidget(self.hasWCS)
        self.hasDistortion = QtWidgets.QLabel(self.attributsgroup)
        self.hasDistortion.setEnabled(True)
        self.hasDistortion.setObjectName("hasDistortion")
        self.verticalLayout_6.addWidget(self.hasDistortion)
        self.isFlipped = QtWidgets.QLabel(self.attributsgroup)
        self.isFlipped.setObjectName("isFlipped")
        self.verticalLayout_6.addWidget(self.isFlipped)
        spacerItem1 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem1)
        self.verticalLayout_6.setStretch(0, 1)
        self.verticalLayout_6.setStretch(4, 1)
        self.horizontalLayout_5.addWidget(self.attributsgroup)
        self.line_3 = QtWidgets.QFrame(ImageDialog)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout_5.addWidget(self.line_3)
        self.imagesettings = QtWidgets.QGroupBox(ImageDialog)
        self.imagesettings.setObjectName("imagesettings")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.imagesettings)
        self.verticalLayout.setContentsMargins(4, 6, 4, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.stretch = QtWidgets.QComboBox(self.imagesettings)
        self.stretch.setMinimumSize(QtCore.QSize(0, 25))
        self.stretch.setObjectName("stretch")
        self.verticalLayout.addWidget(self.stretch)
        self.color = QtWidgets.QComboBox(self.imagesettings)
        self.color.setMinimumSize(QtCore.QSize(0, 25))
        self.color.setObjectName("color")
        self.verticalLayout.addWidget(self.color)
        self.zoom = QtWidgets.QComboBox(self.imagesettings)
        self.zoom.setMinimumSize(QtCore.QSize(0, 25))
        self.zoom.setObjectName("zoom")
        self.verticalLayout.addWidget(self.zoom)
        self.horizontalLayout_5.addWidget(self.imagesettings)
        spacerItem2 = QtWidgets.QSpacerItem(5, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem2)
        self.horizontalLayout_5.setStretch(7, 1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.line = QtWidgets.QFrame(ImageDialog)
        self.line.setAutoFillBackground(True)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.imagelayout = QtWidgets.QHBoxLayout()
        self.imagelayout.setContentsMargins(2, 2, 2, 2)
        self.imagelayout.setSpacing(2)
        self.imagelayout.setObjectName("imagelayout")
        self.groupBox = QtWidgets.QGroupBox(ImageDialog)
        self.groupBox.setMinimumSize(QtCore.QSize(120, 0))
        self.groupBox.setMaximumSize(QtCore.QSize(100, 16777215))
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setContentsMargins(4, 6, 4, 2)
        self.verticalLayout_3.setSpacing(2)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName("label")
        self.verticalLayout_3.addWidget(self.label)
        self.object = QtWidgets.QLineEdit(self.groupBox)
        self.object.setObjectName("object")
        self.verticalLayout_3.addWidget(self.object)
        self.label_2 = QtWidgets.QLabel(self.groupBox)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.ra = QtWidgets.QLineEdit(self.groupBox)
        self.ra.setObjectName("ra")
        self.verticalLayout_3.addWidget(self.ra)
        self.label_3 = QtWidgets.QLabel(self.groupBox)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_3.addWidget(self.label_3)
        self.dec = QtWidgets.QLineEdit(self.groupBox)
        self.dec.setObjectName("dec")
        self.verticalLayout_3.addWidget(self.dec)
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_3.addWidget(self.label_6)
        self.rotation = QtWidgets.QLineEdit(self.groupBox)
        self.rotation.setObjectName("rotation")
        self.verticalLayout_3.addWidget(self.rotation)
        self.label_7 = QtWidgets.QLabel(self.groupBox)
        self.label_7.setObjectName("label_7")
        self.verticalLayout_3.addWidget(self.label_7)
        self.scale = QtWidgets.QLineEdit(self.groupBox)
        self.scale.setObjectName("scale")
        self.verticalLayout_3.addWidget(self.scale)
        self.label_8 = QtWidgets.QLabel(self.groupBox)
        self.label_8.setObjectName("label_8")
        self.verticalLayout_3.addWidget(self.label_8)
        self.expTime = QtWidgets.QLineEdit(self.groupBox)
        self.expTime.setObjectName("expTime")
        self.verticalLayout_3.addWidget(self.expTime)
        self.label_10 = QtWidgets.QLabel(self.groupBox)
        self.label_10.setObjectName("label_10")
        self.verticalLayout_3.addWidget(self.label_10)
        self.filter = QtWidgets.QLineEdit(self.groupBox)
        self.filter.setObjectName("filter")
        self.verticalLayout_3.addWidget(self.filter)
        self.label_11 = QtWidgets.QLabel(self.groupBox)
        self.label_11.setObjectName("label_11")
        self.verticalLayout_3.addWidget(self.label_11)
        self.binX = QtWidgets.QLineEdit(self.groupBox)
        self.binX.setObjectName("binX")
        self.verticalLayout_3.addWidget(self.binX)
        self.label_12 = QtWidgets.QLabel(self.groupBox)
        self.label_12.setObjectName("label_12")
        self.verticalLayout_3.addWidget(self.label_12)
        self.binY = QtWidgets.QLineEdit(self.groupBox)
        self.binY.setObjectName("binY")
        self.verticalLayout_3.addWidget(self.binY)
        self.label_9 = QtWidgets.QLabel(self.groupBox)
        self.label_9.setObjectName("label_9")
        self.verticalLayout_3.addWidget(self.label_9)
        self.ccdTemp = QtWidgets.QLineEdit(self.groupBox)
        self.ccdTemp.setObjectName("ccdTemp")
        self.verticalLayout_3.addWidget(self.ccdTemp)
        self.label_4 = QtWidgets.QLabel(self.groupBox)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_3.addWidget(self.label_4)
        self.sqm = QtWidgets.QLineEdit(self.groupBox)
        self.sqm.setObjectName("sqm")
        self.verticalLayout_3.addWidget(self.sqm)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem3)
        self.imagelayout.addWidget(self.groupBox)
        self.image = QtWidgets.QWidget(ImageDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.image.sizePolicy().hasHeightForWidth())
        self.image.setSizePolicy(sizePolicy)
        self.image.setMinimumSize(QtCore.QSize(0, 0))
        self.image.setObjectName("image")
        self.imagelayout.addWidget(self.image)
        self.verticalLayout_2.addLayout(self.imagelayout)
        self.verticalLayout_2.setStretch(2, 1)

        self.retranslateUi(ImageDialog)
        QtCore.QMetaObject.connectSlotsByName(ImageDialog)

    def retranslateUi(self, ImageDialog):
        _translate = QtCore.QCoreApplication.translate
        ImageDialog.setWindowTitle(_translate("ImageDialog", "Imaging"))
        self.exposeN.setText(_translate("ImageDialog", "Expose N"))
        self.solve.setText(_translate("ImageDialog", "Solve"))
        self.abortSolve.setText(_translate("ImageDialog", "Abort"))
        self.abortImage.setText(_translate("ImageDialog", "Abort"))
        self.checkAutoSolve.setText(_translate("ImageDialog", "Auto Solve"))
        self.load.setText(_translate("ImageDialog", "Load FITS"))
        self.expose.setText(_translate("ImageDialog", "Expose 1"))
        self.checkStackImages.setText(_translate("ImageDialog", "Stack Images"))
        self.groupBox_3.setTitle(_translate("ImageDialog", "Views / Coords"))
        self.checkUseWCS.setText(_translate("ImageDialog", "Show RA / DEC"))
        self.checkShowGrid.setText(_translate("ImageDialog", "Show grid"))
        self.checkShowCrosshair.setText(_translate("ImageDialog", "Show crosshair  "))
        self.attributsgroup.setTitle(_translate("ImageDialog", "Image attributes"))
        self.hasWCS.setText(_translate("ImageDialog", "WCS Header"))
        self.hasDistortion.setText(_translate("ImageDialog", "Distortion Param"))
        self.isFlipped.setText(_translate("ImageDialog", "Is Flipped"))
        self.imagesettings.setTitle(_translate("ImageDialog", "Image settings"))
        self.groupBox.setTitle(_translate("ImageDialog", "Fits Header"))
        self.label.setText(_translate("ImageDialog", "Object Name"))
        self.label_2.setText(_translate("ImageDialog", "RA [hours]"))
        self.label_3.setText(_translate("ImageDialog", "DEC [°]"))
        self.label_6.setText(_translate("ImageDialog", "Rotation [°]"))
        self.label_7.setText(_translate("ImageDialog", "Scale [arcs/pixel]"))
        self.label_8.setText(_translate("ImageDialog", "Exposure Time [s]"))
        self.label_10.setText(_translate("ImageDialog", "Filter"))
        self.label_11.setText(_translate("ImageDialog", "Binning X"))
        self.label_12.setText(_translate("ImageDialog", "Binning Y"))
        self.label_9.setText(_translate("ImageDialog", "CCD Temp [°]"))
        self.label_4.setText(_translate("ImageDialog", "SQM [mpas]"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    ImageDialog = QtWidgets.QWidget()
    ui = Ui_ImageDialog()
    ui.setupUi(ImageDialog)
    ImageDialog.show()
    sys.exit(app.exec_())



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
from datetime import datetime
# external packages
import PyQt5
import numpy as np
from indibase import qtIndiBase
# local imports


class IndiClass(object):
    """
    the class indiClass inherits all information and handling of indi devices
    this class will be only referenced from other classes and not directly used

        >>> fw = IndiClass(
        >>>                  host=host
        >>>                  name=''
        >>>                 )
    """

    __all__ = ['IndiClass']

    version = '0.100.0'
    logger = logging.getLogger(__name__)

    # update rate to 1 seconds for setting indi server
    UPDATE_RATE = 1
    RETRY_DELAY = 3000

    def __init__(self,
                 host=None,
                 name='',
                 ):
        super().__init__()

        self.client = qtIndiBase.Client(host=host)
        self.name = name
        self.data = {}
        self.device = None

        self.timerRetry = PyQt5.QtCore.QTimer()
        self.timerRetry.setSingleShot(True)
        self.timerRetry.timeout.connect(self.startRetry)

        # link signals
        self.client.signals.newDevice.connect(self.newDevice)
        self.client.signals.removeDevice.connect(self.removeDevice)
        self.client.signals.newProperty.connect(self.connectDevice)
        self.client.signals.newNumber.connect(self.updateNumber)
        self.client.signals.defNumber.connect(self.updateNumber)
        self.client.signals.newSwitch.connect(self.updateSwitch)
        self.client.signals.defSwitch.connect(self.updateSwitch)
        self.client.signals.newText.connect(self.updateText)
        self.client.signals.defText.connect(self.updateText)
        self.client.signals.newLight.connect(self.updateLight)
        self.client.signals.defLight.connect(self.updateLight)
        self.client.signals.newBLOB.connect(self.updateBLOB)
        self.client.signals.defBLOB.connect(self.updateBLOB)
        self.client.signals.deviceConnected.connect(self.setUpdateConfig)
        self.client.signals.serverConnected.connect(self.serverConnected)
        self.client.signals.serverDisconnected.connect(self.serverDisconnected)

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    def serverConnected(self):
        """
        serverConnected is called when the server signals the connection. if ao, we would
        like to start watching the defined device. this will be triggered directly

        :return: success
        """

        if self.name:
            suc = self.client.watchDevice(self.name)
            self.logger.info(f'Indi server {self.name} connected, watch: result:{suc}')
            return suc
        return False

    @staticmethod
    def serverDisconnected():
        return True

    def newDevice(self, deviceName):
        """
        newDevice is called whenever a new device entry is received in indi client. it
        adds the device if the name fits to the given name in configuration.

        :param deviceName:
        :return: true for test purpose
        """

        if deviceName == self.name:
            self.device = self.client.getDevice(deviceName)
        return True

    def removeDevice(self, deviceName):
        """
        removeDevice is called whenever a device is removed from indi client. it sets
        the device entry to None

        :param deviceName:
        :return: true for test purpose
        """

        if deviceName == self.name:
            self.device = None
            self.data = {}
            return True
        else:
            return False

    def startRetry(self):
        """
        startRetry tries to connect the server a second time, if that
        is not the case actually.

        :return: True for test purpose
        """
        if not self.data:
            self.stopCommunication()
            self.startCommunication()
            self.logger.info(f'Indi server {self.name} connection retry')

        return True

    def startCommunication(self):
        """
        startCommunication adds a device on the watch list of the server.

        :return: success of reconnecting to server
        """

        self.client.startTimers()

        # adding a single retry if first connect does not happen after 1 second
        self.timerRetry.start(self.RETRY_DELAY)

        suc = self.client.connectServer()
        return suc

    def stopCommunication(self):
        """
        stopCommunication adds a device on the watch list of the server.

        :return: success of reconnecting to server
        """

        self.client.stopTimers()
        suc = self.client.disconnectServer()
        return suc

    def connectDevice(self, deviceName, propertyName):
        """
        connectDevice is called when a new property is received and checks it against
        property CONNECTION. if this is there, we could check the connection state of
        a given device

        :param deviceName:
        :param propertyName:
        :return: success if device could connect
        """
        if propertyName != 'CONNECTION':
            return False

        suc = False
        if deviceName == self.name:
            suc = self.client.connectDevice(deviceName=deviceName)
        return suc

    def setUpdateConfig(self, deviceName):
        """
        _setUpdateRate corrects the update rate of weather devices to get an defined
        setting regardless, what is setup in server side.

        :param deviceName:
        :return: success
        """
        pass

    def updateNumber(self, deviceName, propertyName):
        """
        updateNumber is called whenever a new number is received in client. it runs
        through the device list and writes the number data to the according locations.

        :param deviceName:
        :param propertyName:
        :return:
        """
        pass

    def updateSwitch(self, deviceName, propertyName):
        """
        updateSwitch is called whenever a new switch is received in client. it runs
        through the device list and writes the number data to the according locations.

        :param deviceName:
        :param propertyName:
        :return:
        """
        pass

    def updateText(self, deviceName, propertyName):
        """
        updateText is called whenever a new text is received in client. it runs
        through the device list and writes the number data to the according locations.

        :param deviceName:
        :param propertyName:
        :return:
        """
        pass

    def updateLight(self, deviceName, propertyName):
        """
        updateLight is called whenever a new light is received in client. it runs
        through the device list and writes the number data to the according locations.

        :param deviceName:
        :param propertyName:
        :return:
        """
        pass

    def updateBLOB(self, deviceName, propertyName):
        """
        updateBLOB is called whenever a new BLOB is received in client. it runs
        through the device list and writes the number data to the according locations.

        :param deviceName:
        :param propertyName:
        :return:
        """
        pass



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
from collections import namedtuple

# we are adding named tuples for the readability of the code
# fist package is related to modeling procedure and defines the model points
# and the necessary structures for storing the data, which is generated during
# the model run

# the model point itself
Point = namedtuple('Point', 'altitude azimuth')

# imaging parameters
IParam = namedtuple('IParam', 'expTime binning subFrame fastReadout')

# modeling parameters
MParam = namedtuple('MParam', 'number count path name astrometry timeout radius')

# modeling generated data for programming the model
MData = namedtuple('MData', 'raMJNow decMJNow raSJNow decSJNow sidereal julian pierside')

# mount retrofit data
RData = namedtuple('RData', 'errorRMS errorRA errorDEC')

# overall structure containing all of the above
MPoint = namedtuple('MPoint', 'mParam iParam point mData rData')


# second cluster is for the astrometry stuff and solving data
# during solving we extract some data from headers and store them

# solving values
Solve = namedtuple('Solve', 'raJ2000 decJ2000 angle scale error flipped path')

# overall structure which as well add the solve success part
Solution = namedtuple('Solution', 'success solve')



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
import pickle
from io import BytesIO
# external packages
import PyQt5.QtCore
import PyQt5.QtWidgets
import PyQt5.uic
# This import registers the 3D projection, but is otherwise unused.
from mpl_toolkits.mplot3d import Axes3D
from mpl_toolkits.mplot3d.art3d import Poly3DCollection
import matplotlib.pyplot as plt
import numpy as np
from skyfield.api import EarthSatellite
# local import
from mw4.gui import widget
from mw4.gui.widgets import satellite_ui
from mw4.base import transform
from mw4.base.tpool import Worker


class SatelliteWindowSignals(PyQt5.QtCore.QObject):
    """
    The CameraSignals class offers a list of signals to be used and instantiated by
    the Mount class to get signals for triggers for finished tasks to
    enable a gui to update their values transferred to the caller back.

    This has to be done in a separate class as the signals have to be subclassed from
    QObject and the Mount class itself is subclassed from object
    """

    __all__ = ['SatelliteWindowSignals']
    version = '0.1'

    show = PyQt5.QtCore.pyqtSignal(object)
    update = PyQt5.QtCore.pyqtSignal(object, object, object)


class SatelliteWindow(widget.MWidget):
    """
    the satellite window class handles

    """

    __all__ = ['SatelliteWindow',
               'receiveSatelliteAndShow',
               'updatePositions',
               ]
    version = '0.100.0'
    logger = logging.getLogger(__name__)

    # length of forecast time in hours
    FORECAST_TIME = 3
    # earth radius
    EARTH_RADIUS = 6378.0

    def __init__(self, app, threadPool):
        super().__init__()
        self.app = app
        self.threadPool = threadPool

        self.ui = satellite_ui.Ui_SatelliteDialog()
        self.ui.setupUi(self)
        self.initUI()
        self.signals = SatelliteWindowSignals()
        self.satellite = None
        self.plotSatPosSphere1 = None
        self.plotSatPosSphere2 = None
        self.plotSatPosHorizon = None
        self.plotSatPosEarth = None

        self.satSphereMat1 = self.embedMatplot(self.ui.satSphere1)
        self.satSphereMat1.parentWidget().setStyleSheet(self.BACK_BG)
        self.satSphereMat2 = self.embedMatplot(self.ui.satSphere2)
        self.satSphereMat2.parentWidget().setStyleSheet(self.BACK_BG)
        self.satHorizonMat = self.embedMatplot(self.ui.satHorizon)
        self.satHorizonMat.parentWidget().setStyleSheet(self.BACK_BG)
        self.satEarthMat = self.embedMatplot(self.ui.satEarth)
        self.satEarthMat.parentWidget().setStyleSheet(self.BACK_BG)

        self.signals.show.connect(self.receiveSatelliteAndShow)
        self.signals.update.connect(self.updatePositions)

        # as we cannot access data from Qt resource system, we have to convert it to
        # ByteIO first
        stream = PyQt5.QtCore.QFile(':/worldmap.dat')
        stream.open(PyQt5.QtCore.QFile.ReadOnly)
        pickleData = stream.readAll()
        stream.close()
        # loading the world image from nasa as PNG as matplotlib only loads png.
        self.world = pickle.load(BytesIO(pickleData))

        self.initConfig()
        self.showWindow()

    def initConfig(self):
        """
        initConfig read the key out of the configuration dict and stores it to the gui
        elements. if some initialisations have to be proceeded with the loaded persistent
        data, they will be launched as well in this method.

        :return: True for test purpose
        """

        if 'satelliteW' not in self.app.config:
            self.app.config['satelliteW'] = {}
        config = self.app.config['satelliteW']
        x = config.get('winPosX', 100)
        y = config.get('winPosY', 100)
        if x > self.screenSizeX:
            x = 0
        if y > self.screenSizeY:
            y = 0
        self.move(x, y)
        height = config.get('height', 600)
        width = config.get('width', 800)
        self.resize(width, height)

        return True

    def storeConfig(self):
        """
        storeConfig writes the keys to the configuration dict and stores. if some
        saving has to be proceeded to persistent data, they will be launched as
        well in this method.

        :return: True for test purpose
        """
        if 'satelliteW' not in self.app.config:
            self.app.config['satelliteW'] = {}
        config = self.app.config['satelliteW']
        config['winPosX'] = self.pos().x()
        config['winPosY'] = self.pos().y()
        config['height'] = self.height()
        config['width'] = self.width()

        return True

    def closeEvent(self, closeEvent):
        self.storeConfig()

        # gui signals
        super().closeEvent(closeEvent)

    def showWindow(self):
        self.receiveSatelliteAndShow(self.app.mainW.satellite)
        self.show()

    def receiveSatelliteAndShow(self, satellite=None):
        """
        receiveSatelliteAndShow receives a signal with the content of the selected satellite.
        it locally sets it an draws the the complete view

        :param satellite:
        :return: true for test purpose
        """

        if satellite is None:
            self.drawSatellite()
            return False

        self.satellite = satellite
        if satellite.model.no < 0.02:
            self.FORECAST_TIME = 24
        else:
            self.FORECAST_TIME = 3
        self.drawSatellite()

        return True

    def updatePositions(self, observe=None, subpoint=None, altaz=None):
        """
        updatePositions is triggered once a second and update the satellite position in each
        view.

        :return: success
        """
        if observe is None:
            return False
        if subpoint is None:
            return False
        if altaz is None:
            return False

        if self.satellite is None:
            return False
        if self.plotSatPosEarth is None:
            return False
        if self.plotSatPosHorizon is None:
            return False
        if self.plotSatPosSphere1 is None:
            return False
        if self.plotSatPosSphere2 is None:
            return False

        # sphere1
        x, y, z = observe.position.km
        self.plotSatPosSphere1.set_data_3d((x, y, z))

        # sphere2
        lat = subpoint.latitude.radians
        lon = subpoint.longitude.radians
        elev = subpoint.elevation.m / 1000 + self.EARTH_RADIUS
        x, y, z = transform.sphericalToCartesian(azimuth=lon,
                                                 altitude=lat,
                                                 radius=elev)
        self.plotSatPosSphere2.set_data_3d((x, y, z))

        # earth
        lat = subpoint.latitude.degrees
        lon = subpoint.longitude.degrees
        self.plotSatPosEarth.set_data((lon, lat))

        # horizon
        alt, az, _ = altaz
        alt = alt.degrees
        az = az.degrees
        self.plotSatPosHorizon.set_data((az, alt))

        # update the plot and redraw
        self.satSphereMat1.figure.canvas.draw()
        self.satEarthMat.figure.canvas.draw()
        self.satHorizonMat.figure.canvas.draw()

        return True

    @staticmethod
    def makeCubeLimits(axe, centers=None, hw=None):
        """

        :param axe:
        :param centers:
        :param hw:
        :return:
        """

        limits = axe.get_xlim(), axe.get_ylim(), axe.get_zlim()
        if centers is None:
            centers = [0.5 * sum(pair) for pair in limits]

        if hw is None:
            widths = [pair[1] - pair[0] for pair in limits]
            hw = 0.5 * max(widths)
            axe.set_xlim(centers[0] - hw, centers[0] + hw)
            axe.set_ylim(centers[1] - hw, centers[1] + hw)
            axe.set_zlim(centers[2] - hw, centers[2] + hw)

        else:
            try:
                hwx, hwy, hwz = hw
                axe.set_xlim(centers[0] - hwx, centers[0] + hwx)
                axe.set_ylim(centers[1] - hwy, centers[1] + hwy)
                axe.set_zlim(centers[2] - hwz, centers[2] + hwz)
            except Exception:
                axe.set_xlim(centers[0] - hw, centers[0] + hw)
                axe.set_ylim(centers[1] - hw, centers[1] + hw)
                axe.set_zlim(centers[2] - hw, centers[2] + hw)

        return centers, hw

    def drawSphere1(self, observe=None):
        """
        draw sphere and put face color als image overlay:

        https://stackoverflow.com/questions/53074908/
        map-an-image-onto-a-sphere-and-plot-3d-trajectories

        but performance problems

        see also:

        https://space.stackexchange.com/questions/25958/
        how-can-i-plot-a-satellites-orbit-in-3d-from-a-tle-using-python-and-skyfield

        :param observe:
        :return: success
        """

        figure = self.satSphereMat1.figure
        figure.clf()
        figure.subplots_adjust(left=-0.1, right=1.1, bottom=-0.3, top=1.3)
        axe = figure.add_subplot(111, projection='3d')

        # switching all visual grids and planes off
        axe.set_facecolor((0, 0, 0, 0))
        axe.tick_params(colors=self.M_GREY,
                        labelsize=12)
        axe.set_axis_off()
        axe.w_xaxis.set_pane_color((0.0, 0.0, 0.0, 0.0))
        axe.w_yaxis.set_pane_color((0.0, 0.0, 0.0, 0.0))
        axe.w_zaxis.set_pane_color((0.0, 0.0, 0.0, 0.0))

        # calculating sphere
        theta = np.linspace(0, 2 * np.pi, 51)
        cth, sth, zth = [f(theta) for f in [np.cos, np.sin, np.zeros_like]]
        lon0 = self.EARTH_RADIUS * np.vstack((cth, zth, sth))
        longitudes = []
        lonBase = np.arange(-180, 180, 15)
        for phi in np.radians(lonBase):
            cph, sph = [f(phi) for f in [np.cos, np.sin]]
            lon = np.vstack((lon0[0] * cph - lon0[1] * sph,
                             lon0[1] * cph + lon0[0] * sph,
                             lon0[2]))
            longitudes.append(lon)
        lats = []
        latBase = np.arange(-75, 90, 15)
        for phi in np.radians(latBase):
            cph, sph = [f(phi) for f in [np.cos, np.sin]]
            lat = self.EARTH_RADIUS * np.vstack((cth * cph, sth * cph, zth + sph))
            lats.append(lat)

        # plotting sphere and labels
        for i, longitude in enumerate(longitudes):
            x, y, z = longitude
            axe.plot(x, y, z, '-k', lw=1,
                     color=self.M_GREY)
        for i, lat in enumerate(lats):
            x, y, z = lat
            axe.plot(x, y, z, '-k', lw=1,
                     color=self.M_GREY)

        axe.plot([0, 0],
                 [0, 0],
                 [- self.EARTH_RADIUS * 1.1, self.EARTH_RADIUS * 1.1],
                 lw=3,
                 color=self.M_BLUE)
        axe.text(0, 0, self.EARTH_RADIUS * 1.2, 'N',
                 fontsize=14,
                 color=self.M_BLUE)
        axe.text(0, 0, - self.EARTH_RADIUS * 1.2 - 200, 'S',
                 fontsize=14,
                 color=self.M_BLUE)

        # empty chart if no satellite is chosen
        if observe is None:
            axe.figure.canvas.draw()
            return False

        # drawing satellite
        x, y, z = observe.position.km
        axe.plot(x, y, z, color=self.M_YELLOW)

        self.plotSatPosSphere1, = axe.plot([x[0]], [y[0]], [z[0]],
                                           marker='o',
                                           markersize=10,
                                           color=self.M_PINK)
        self.makeCubeLimits(axe)
        axe.figure.canvas.draw()
        return True

    def drawSphere2(self, observe=None, subpoint=None):
        """
        draw sphere and put face color als image overlay:

        https://stackoverflow.com/questions/53074908/
        map-an-image-onto-a-sphere-and-plot-3d-trajectories

        but performance problems

        see also:

        https://space.stackexchange.com/questions/25958/
        how-can-i-plot-a-satellites-orbit-in-3d-from-a-tle-using-python-and-skyfield

        :param observe:
        :param subpoint:
        :return: success
        """

        figure = self.satSphereMat2.figure
        figure.clf()
        figure.subplots_adjust(left=-0.1, right=1.1, bottom=-0.3, top=1.3)
        axe = figure.add_subplot(111, projection='3d')

        # switching all visual grids and planes off
        axe.set_facecolor((0, 0, 0, 0))
        axe.tick_params(colors=self.M_GREY,
                        labelsize=12)
        axe.set_axis_off()
        axe.w_xaxis.set_pane_color((0.0, 0.0, 0.0, 0.0))
        axe.w_yaxis.set_pane_color((0.0, 0.0, 0.0, 0.0))
        axe.w_zaxis.set_pane_color((0.0, 0.0, 0.0, 0.0))

        # calculating sphere
        theta = np.linspace(0, 2 * np.pi, 51)
        cth, sth, zth = [f(theta) for f in [np.cos, np.sin, np.zeros_like]]
        lon0 = self.EARTH_RADIUS * np.vstack((cth, zth, sth))
        longitudes = []
        lonBase = np.arange(-180, 180, 15)
        for phi in np.radians(lonBase):
            cph, sph = [f(phi) for f in [np.cos, np.sin]]
            lon = np.vstack((lon0[0] * cph - lon0[1] * sph,
                             lon0[1] * cph + lon0[0] * sph,
                             lon0[2]))
            longitudes.append(lon)
        lats = []
        latBase = np.arange(-75, 90, 15)
        for phi in np.radians(latBase):
            cph, sph = [f(phi) for f in [np.cos, np.sin]]
            lat = self.EARTH_RADIUS * np.vstack((cth * cph, sth * cph, zth + sph))
            lats.append(lat)

        # plotting sphere and labels
        for i, longitude in enumerate(longitudes):
            x, y, z = longitude
            axe.plot(x, y, z, '-k', lw=1,
                     color=self.M_GREY)
        for i, lat in enumerate(lats):
            x, y, z = lat
            axe.plot(x, y, z, '-k', lw=1,
                     color=self.M_GREY)

        axe.plot([0, 0],
                 [0, 0],
                 [- self.EARTH_RADIUS * 1.1, self.EARTH_RADIUS * 1.1],
                 lw=3,
                 color=self.M_BLUE)
        axe.text(0, 0, self.EARTH_RADIUS * 1.2, 'N',
                 fontsize=14,
                 color=self.M_BLUE)
        axe.text(0, 0, - self.EARTH_RADIUS * 1.2 - 200, 'S',
                 fontsize=14,
                 color=self.M_BLUE)

        # plot world
        for key in self.world.keys():
            shape = self.world[key]
            x, y, z = transform.sphericalToCartesian(azimuth=shape['xRad'],
                                                     altitude=shape['yRad'],
                                                     radius=self.EARTH_RADIUS)
            verts = [list(zip(x, y, z))]
            collect = Poly3DCollection(verts, facecolors=self.M_BLUE, alpha=0.5)
            axe.add_collection3d(collect)

        # drawing home position location on earth
        lat = self.app.mount.obsSite.location.latitude.radians
        lon = self.app.mount.obsSite.location.longitude.radians
        x, y, z = transform.sphericalToCartesian(altitude=lat,
                                                 azimuth=lon,
                                                 radius=self.EARTH_RADIUS)
        axe.plot([x], [y], [z],
                 marker='.',
                 markersize=12,
                 color=self.M_RED,
                 )

        # empty chart if no satellite is chosen
        if observe is None:
            axe.figure.canvas.draw()
            return False

        # drawing satellite subpoint path
        lat = subpoint.latitude.radians
        lon = subpoint.longitude.radians
        elev = subpoint.elevation.m / 1000 + self.EARTH_RADIUS
        x, y, z = transform.sphericalToCartesian(azimuth=lon,
                                                 altitude=lat,
                                                 radius=elev)
        axe.plot(x, y, z, color=self.M_YELLOW)

        # draw satellite position
        self.plotSatPosSphere2, = axe.plot([x[0]], [y[0]], [z[0]],
                                           marker='o',
                                           markersize=10,
                                           color=self.M_PINK)

        # finalizing
        self.makeCubeLimits(axe)
        axe.figure.canvas.draw()

        return True

    def drawEarth(self, subpoint=None):
        """
        drawEarth show a full earth view with the path of the subpoint of the satellite
        drawn on it.

        :param subpoint:
        :return: success
        """

        figure = self.satEarthMat.figure
        figure.clf()
        figure.subplots_adjust(left=0.2, right=0.85, bottom=0.2, top=0.9)
        axe = self.satEarthMat.figure.add_subplot(1, 1, 1, facecolor=None)

        axe.set_facecolor((0, 0, 0, 0))
        axe.set_xlim(-180, 180)
        axe.set_ylim(-90, 90)
        axe.spines['bottom'].set_color(self.M_BLUE)
        axe.spines['top'].set_color(self.M_BLUE)
        axe.spines['left'].set_color(self.M_BLUE)
        axe.spines['right'].set_color(self.M_BLUE)
        axe.grid(True, color=self.M_GREY)
        axe.tick_params(axis='x',
                        colors=self.M_BLUE,
                        labelsize=12)
        axe.set_xticks(np.arange(-180, 181, 45))
        axe.tick_params(axis='y',
                        colors=self.M_BLUE,
                        which='both',
                        labelleft=True,
                        labelright=True,
                        labelsize=12)
        axe.set_xlabel('Longitude in degrees',
                       color=self.M_BLUE,
                       fontweight='bold',
                       fontsize=12)
        axe.set_ylabel('Latitude in degrees',
                       color=self.M_BLUE,
                       fontweight='bold',
                       fontsize=12)

        # plot world
        for key in self.world.keys():
            shape = self.world[key]
            axe.fill(shape['xDeg'], shape['yDeg'], color=self.M_BLUE, alpha=0.2)
            axe.plot(shape['xDeg'], shape['yDeg'], color=self.M_BLUE, lw=1, alpha=0.4)

        # mark the site location in the map
        lat = self.app.mount.obsSite.location.latitude.degrees
        lon = self.app.mount.obsSite.location.longitude.degrees
        axe.plot(lon,
                 lat,
                 marker='.',
                 markersize=10,
                 color=self.M_RED)

        # empty chart if no satellite is chosen
        if subpoint is None:
            axe.figure.canvas.draw()
            return False

        # drawing satellite subpoint path
        lat = subpoint.latitude.degrees
        lon = subpoint.longitude.degrees

        axe.plot(lon,
                 lat,
                 marker='o',
                 markersize=1,
                 linestyle='none',
                 color=self.M_YELLOW)

        # show the actual position
        self.plotSatPosEarth, = axe.plot(lon[0],
                                         lat[0],
                                         marker='o',
                                         markersize=10,
                                         linestyle='none',
                                         color=self.M_PINK)
        axe.figure.canvas.draw()
        return True

    def staticHorizon(self, axes=None):
        """

        :param axes: matplotlib axes object
        :return:
        """

        if not self.app.data.horizonP:
            return False

        alt, az = zip(*self.app.data.horizonP)

        axes.fill(az, alt, color=self.M_GREEN, zorder=-20, alpha=0.2)
        axes.plot(az, alt, color=self.M_GREEN, zorder=-20, lw=2, alpha=0.4)

        return True

    def drawHorizonView(self, difference=None):
        """
        drawHorizonView shows the horizon and enable the users to explore a satellite
        passing by

        :param difference:
        :return: success
        """

        figure = self.satHorizonMat.figure
        figure.clf()
        figure.subplots_adjust(left=0.2, right=0.85, bottom=0.2, top=0.9)
        axe = self.satHorizonMat.figure.add_subplot(1, 1, 1, facecolor=None)

        # add horizon limit if selected
        self.staticHorizon(axes=axe)

        axe.set_facecolor((0, 0, 0, 0))
        axe.set_xlim(0, 360)
        axe.set_ylim(0, 90)
        axe.spines['bottom'].set_color(self.M_BLUE)
        axe.spines['top'].set_color(self.M_BLUE)
        axe.spines['left'].set_color(self.M_BLUE)
        axe.spines['right'].set_color(self.M_BLUE)
        axe.grid(True, color=self.M_GREY)
        axe.tick_params(axis='x',
                        colors=self.M_BLUE,
                        labelsize=12)
        axe.set_xticks(np.arange(0, 361, 45))
        axe.tick_params(axis='y',
                        colors=self.M_BLUE,
                        which='both',
                        labelleft=True,
                        labelright=True,
                        labelsize=12)
        axe.set_xlabel('Azimuth in degrees',
                       color=self.M_BLUE,
                       fontweight='bold',
                       fontsize=12)
        axe.set_ylabel('Altitude in degrees',
                       color=self.M_BLUE,
                       fontweight='bold',
                       fontsize=12)

        axe.text(0, 1, 'N',
                 fontsize=14,
                 color=self.M_BLUE)

        axe.text(85, 1, 'E',
                 fontsize=14,
                 color=self.M_BLUE)

        axe.text(175, 1, 'S',
                 fontsize=14,
                 color=self.M_BLUE)

        axe.text(265, 1, 'W',
                 fontsize=14,
                 color=self.M_BLUE)

        axe.text(345, 1, 'N',
                 fontsize=14,
                 color=self.M_BLUE)

        # empty chart if no satellite is chosen
        if difference is None:
            axe.figure.canvas.draw()
            return False

        # orbital calculations
        alt, az, _ = difference.altaz()
        alt = alt.degrees
        az = az.degrees

        # draw path
        axe.plot(az,
                 alt,
                 marker='o',
                 markersize=1,
                 linestyle='none',
                 color=self.M_YELLOW)

        # draw actual position
        self.plotSatPosHorizon, = axe.plot(az[0],
                                           alt[0],
                                           marker='o',
                                           markersize=10,
                                           linestyle='none',
                                           color=self.M_PINK)

        axe.figure.canvas.draw()
        return True

    def drawSatellite(self):
        """
        drawSatellite draws 3 different views of the actual satellite situation: a sphere
        a horizon view and an earth view.

        :return: True for test purpose
        """

        timescale = self.app.mount.obsSite.ts
        forecast = np.arange(0, self.FORECAST_TIME, 0.005 * self.FORECAST_TIME / 3) / 24
        now = timescale.now()
        timeVector = timescale.tt_jd(now.tt + forecast)

        if self.satellite is not None:
            observe = self.satellite.at(timeVector)
            subpoint = observe.subpoint()
            difference = (self.satellite - self.app.mount.obsSite.location).at(timeVector)
        else:
            observe = subpoint = difference = None

        self.drawSphere1(observe=observe)
        self.drawSphere2(observe=observe, subpoint=subpoint)
        self.drawEarth(subpoint=subpoint)
        self.drawHorizonView(difference=difference)

        return True



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
from datetime import datetime
# external packages
import numpy as np
# local imports
from mw4.base import indiClass


class Environ(indiClass.IndiClass):
    """
    the class Environ inherits all information and handling of the environment device

        >>> fw = Environ(
        >>>                  host=host
        >>>                  name=''
        >>>                 )
    """

    __all__ = ['Environ',
               ]

    version = '0.100.0'
    logger = logging.getLogger(__name__)

    # update rate to 1 seconds for setting indi server
    UPDATE_RATE = 1

    def __init__(self,
                 host=None,
                 name='',
                 ):
        super().__init__(host=host,
                         name=name
                         )

    def setUpdateConfig(self, deviceName):
        """
        _setUpdateRate corrects the update rate of weather devices to get an defined
        setting regardless, what is setup in server side.

        :param deviceName:
        :return: success
        """

        if deviceName != self.name:
            return False

        if self.device is None:
            return False

        update = self.device.getNumber('WEATHER_UPDATE')

        if 'PERIOD' not in update:
            return False

        if update.get('PERIOD', 0) == self.UPDATE_RATE:
            return True

        update['PERIOD'] = self.UPDATE_RATE
        suc = self.client.sendNewNumber(deviceName=deviceName,
                                        propertyName='WEATHER_UPDATE',
                                        elements=update)
        return suc

    @staticmethod
    def _getDewPoint(tempAir, relativeHumidity):
        """
        Compute the dew point in degrees Celsius

        :param tempAir: current ambient temperature in degrees Celsius
        :param relativeHumidity: relative humidity in %
        :return: the dew point in degrees Celsius
        """

        if tempAir < -40 or tempAir > 80:
            return 0
        if relativeHumidity < 0 or relativeHumidity > 100:
            return 0

        A = 17.27
        B = 237.7
        alpha = ((A * tempAir) / (B + tempAir)) + np.log(relativeHumidity / 100.0)
        dewPoint = (B * alpha) / (A - alpha)
        return dewPoint

    def updateNumber(self, deviceName, propertyName):
        """
        updateNumber is called whenever a new number is received in client. it runs
        through the device list and writes the number data to the according locations.
        for global weather data as there is no dew point value available, it calculates
        it and stores it as value as well.

        if no dew point is available in data, it will calculate this value from
        temperature and humidity.

        :param deviceName:
        :param propertyName:
        :return:
        """

        if self.device is None:
            return False
        if deviceName != self.name:
            return False

        for element, value in self.device.getNumber(propertyName).items():

            # consolidate to WEATHER_PRESSURE
            if element == 'WEATHER_BAROMETER':
                key = 'WEATHER_PRESSURE'
            else:
                key = element

            self.data[key] = value
            elArray = key + '_ARRAY'
            elTime = key + '_TIME'
            if elArray not in self.data:
                self.data[elArray] = np.full(100, value)
                self.data[elTime] = np.full(100, datetime.now())
            else:
                self.data[elArray] = np.roll(self.data[elArray], 1)
                self.data[elArray][0] = value
                self.data[elTime] = np.roll(self.data[elTime], 1)
                self.data[elTime][0] = datetime.now()

        if 'WEATHER_DEWPOINT' in self.data:
            return True
        if 'WEATHER_TEMPERATURE' not in self.data:
            return False
        if 'WEATHER_HUMIDITY' not in self.data:
            return False

        temp = self.data['WEATHER_TEMPERATURE']
        humidity = self.data['WEATHER_HUMIDITY']
        dewPoint = self._getDewPoint(temp, humidity)
        self.data['WEATHER_DEWPOINT'] = dewPoint

        return True

    def getFilteredRefracParams(self):
        """
        getFilteredRefracParams filters local temperature and pressure with and moving
        average filter over 5 minutes and returns the filtered values.

        :return:  temperature and pressure
        """

        isTemperature = 'WEATHER_TEMPERATURE_ARRAY' in self.data
        isPressure = 'WEATHER_PRESSURE_ARRAY' in self.data
        if isTemperature and isPressure:
            temp = np.mean(self.data['WEATHER_TEMPERATURE_ARRAY'][:10])
            press = np.mean(self.data['WEATHER_PRESSURE_ARRAY'][:10])
            return temp, press
        else:
            return None, None



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../mw4/gui/widgets/measure.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MeasureDialog(object):
    def setupUi(self, MeasureDialog):
        MeasureDialog.setObjectName("MeasureDialog")
        MeasureDialog.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MeasureDialog.sizePolicy().hasHeightForWidth())
        MeasureDialog.setSizePolicy(sizePolicy)
        MeasureDialog.setMinimumSize(QtCore.QSize(800, 600))
        MeasureDialog.setMaximumSize(QtCore.QSize(16777215, 16777215))
        MeasureDialog.setSizeIncrement(QtCore.QSize(10, 10))
        MeasureDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        MeasureDialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(MeasureDialog)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(4, 6, 4, 6)
        self.horizontalLayout.setSpacing(4)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.measureGroup = QtWidgets.QGroupBox(MeasureDialog)
        self.measureGroup.setObjectName("measureGroup")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.measureGroup)
        self.horizontalLayout_2.setContentsMargins(4, 6, 4, 4)
        self.horizontalLayout_2.setSpacing(4)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.measureSet1 = QtWidgets.QComboBox(self.measureGroup)
        self.measureSet1.setMinimumSize(QtCore.QSize(150, 25))
        self.measureSet1.setObjectName("measureSet1")
        self.horizontalLayout_2.addWidget(self.measureSet1)
        self.measureSet2 = QtWidgets.QComboBox(self.measureGroup)
        self.measureSet2.setMinimumSize(QtCore.QSize(150, 25))
        self.measureSet2.setObjectName("measureSet2")
        self.horizontalLayout_2.addWidget(self.measureSet2)
        self.measureSet3 = QtWidgets.QComboBox(self.measureGroup)
        self.measureSet3.setMinimumSize(QtCore.QSize(150, 25))
        self.measureSet3.setObjectName("measureSet3")
        self.horizontalLayout_2.addWidget(self.measureSet3)
        self.horizontalLayout.addWidget(self.measureGroup)
        self.line_2 = QtWidgets.QFrame(MeasureDialog)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_2.setObjectName("line_2")
        self.horizontalLayout.addWidget(self.line_2)
        self.timeGroup = QtWidgets.QGroupBox(MeasureDialog)
        self.timeGroup.setObjectName("timeGroup")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.timeGroup)
        self.horizontalLayout_3.setContentsMargins(4, 6, 4, 4)
        self.horizontalLayout_3.setSpacing(4)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.timeSet = QtWidgets.QComboBox(self.timeGroup)
        self.timeSet.setMinimumSize(QtCore.QSize(150, 25))
        self.timeSet.setObjectName("timeSet")
        self.horizontalLayout_3.addWidget(self.timeSet)
        self.horizontalLayout.addWidget(self.timeGroup)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.line = QtWidgets.QFrame(MeasureDialog)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setLineWidth(2)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout.addWidget(self.line)
        self.measure = QtWidgets.QWidget(MeasureDialog)
        self.measure.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.measure.sizePolicy().hasHeightForWidth())
        self.measure.setSizePolicy(sizePolicy)
        self.measure.setMinimumSize(QtCore.QSize(0, 0))
        self.measure.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.measure.setSizeIncrement(QtCore.QSize(10, 10))
        self.measure.setBaseSize(QtCore.QSize(10, 10))
        self.measure.setAutoFillBackground(True)
        self.measure.setStyleSheet("")
        self.measure.setObjectName("measure")
        self.verticalLayout.addWidget(self.measure)
        self.verticalLayout.setStretch(2, 1)

        self.retranslateUi(MeasureDialog)
        QtCore.QMetaObject.connectSlotsByName(MeasureDialog)

    def retranslateUi(self, MeasureDialog):
        _translate = QtCore.QCoreApplication.translate
        MeasureDialog.setWindowTitle(_translate("MeasureDialog", "Measurements"))
        self.measureGroup.setTitle(_translate("MeasureDialog", "Measurement sets to be displayed:"))
        self.timeGroup.setTitle(_translate("MeasureDialog", "Measurement windows shows:"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    MeasureDialog = QtWidgets.QWidget()
    ui = Ui_MeasureDialog()
    ui.setupUi(MeasureDialog)
    MeasureDialog.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '../mw4/gui/widgets/satellite.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_SatelliteDialog(object):
    def setupUi(self, SatelliteDialog):
        SatelliteDialog.setObjectName("SatelliteDialog")
        SatelliteDialog.resize(800, 600)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(SatelliteDialog.sizePolicy().hasHeightForWidth())
        SatelliteDialog.setSizePolicy(sizePolicy)
        SatelliteDialog.setMinimumSize(QtCore.QSize(800, 600))
        SatelliteDialog.setMaximumSize(QtCore.QSize(1600, 1200))
        SatelliteDialog.setSizeIncrement(QtCore.QSize(10, 10))
        SatelliteDialog.setBaseSize(QtCore.QSize(10, 10))
        font = QtGui.QFont()
        font.setFamily("Arial")
        SatelliteDialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(SatelliteDialog)
        self.verticalLayout.setContentsMargins(4, 4, 4, 4)
        self.verticalLayout.setSpacing(4)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setContentsMargins(0, 0, -1, -1)
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_4 = QtWidgets.QLabel(SatelliteDialog)
        self.label_4.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_2.addWidget(self.label_4)
        self.satSphere2 = QtWidgets.QWidget(SatelliteDialog)
        self.satSphere2.setObjectName("satSphere2")
        self.verticalLayout_2.addWidget(self.satSphere2)
        self.line = QtWidgets.QFrame(SatelliteDialog)
        self.line.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line.setMidLineWidth(1)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.label_3 = QtWidgets.QLabel(SatelliteDialog)
        self.label_3.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.satSphere1 = QtWidgets.QWidget(SatelliteDialog)
        self.satSphere1.setObjectName("satSphere1")
        self.verticalLayout_2.addWidget(self.satSphere1)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.line_3 = QtWidgets.QFrame(SatelliteDialog)
        self.line_3.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_3.setMidLineWidth(1)
        self.line_3.setFrameShape(QtWidgets.QFrame.VLine)
        self.line_3.setObjectName("line_3")
        self.horizontalLayout_3.addWidget(self.line_3)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_2 = QtWidgets.QLabel(SatelliteDialog)
        self.label_2.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_3.addWidget(self.label_2)
        self.satEarth = QtWidgets.QWidget(SatelliteDialog)
        self.satEarth.setObjectName("satEarth")
        self.verticalLayout_3.addWidget(self.satEarth)
        self.line_2 = QtWidgets.QFrame(SatelliteDialog)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Plain)
        self.line_2.setMidLineWidth(1)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setObjectName("line_2")
        self.verticalLayout_3.addWidget(self.line_2)
        self.label = QtWidgets.QLabel(SatelliteDialog)
        self.label.setMaximumSize(QtCore.QSize(16777215, 25))
        font = QtGui.QFont()
        font.setPointSize(10)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout_3.addWidget(self.label)
        self.satHorizon = QtWidgets.QWidget(SatelliteDialog)
        self.satHorizon.setObjectName("satHorizon")
        self.verticalLayout_3.addWidget(self.satHorizon)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.verticalLayout.setStretch(0, 1)

        self.retranslateUi(SatelliteDialog)
        QtCore.QMetaObject.connectSlotsByName(SatelliteDialog)

    def retranslateUi(self, SatelliteDialog):
        _translate = QtCore.QCoreApplication.translate
        SatelliteDialog.setWindowTitle(_translate("SatelliteDialog", "Satellite"))
        self.label_4.setText(_translate("SatelliteDialog", "   Satellite orbit in relation to fixed observer"))
        self.label_3.setText(_translate("SatelliteDialog", "   Satellite orbit in relation to turning earth"))
        self.label_2.setText(_translate("SatelliteDialog", "   Satellite path over earth surface (subpoints)"))
        self.label.setText(_translate("SatelliteDialog", "   Satellite path over horizon from observers location"))


if __name__ == "__main__":
    import sys
    app = QtWidgets.QApplication(sys.argv)
    SatelliteDialog = QtWidgets.QWidget()
    ui = Ui_SatelliteDialog()
    ui.setupUi(SatelliteDialog)
    SatelliteDialog.show()
    sys.exit(app.exec_())



############################################################
# -*- coding: utf-8 -*-
#
#       #   #  #   #   #    #
#      ##  ##  #  ##  #    #
#     # # # #  # # # #    #  #
#    #  ##  #  ##  ##    ######
#   #   #   #  #   #       #
#
# Python-based Tool for interaction with the 10micron mounts
# GUI with PyQT5 for python
# Python  v3.7.4
#
# Michael Würtenberger
# (c) 2019
#
# Licence APL2.0
#
###########################################################
# standard libraries
import logging
import subprocess
import os
import glob
import re
import fnmatch
import platform
import time
from collections import namedtuple
# external packages
import PyQt5.QtWidgets
from mw4.base import transform
from skyfield.api import Angle
from astropy.io import fits
from astropy.wcs import WCS
import astropy.wcs
from PyQt5.QtTest import QTest
import numpy as np
# local imports
from mw4.base import tpool
from mw4.definitions import Solution, Solve


class AstrometrySignals(PyQt5.QtCore.QObject):
    """
    The AstrometrySignals class offers a list of signals to be used and instantiated by the
    Worker class to get signals for error, finished and result to be transferred to the
    caller back
    """

    __all__ = ['AstrometrySignals']
    version = '1.0.0'

    done = PyQt5.QtCore.pyqtSignal(object)
    result = PyQt5.QtCore.pyqtSignal(object)
    message = PyQt5.QtCore.pyqtSignal(object)


class Astrometry(object):
    """
    the class Astrometry inherits all information and handling of astrometry.net handling

    Keyword definitions could be found under
        https://fits.gsfc.nasa.gov/fits_dictionary.html

        >>> astrometry = Astrometry(app=app,
        >>>                         tempDir=tempDir,
        >>>                         threadPool=threadpool
        >>>                         )

    """

    __all__ = ['Astrometry',
               'solve',
               'solveThreading',
               'checkAvailability',
               ]

    version = '0.100.0'
    logger = logging.getLogger(__name__)

    def __init__(self, app, tempDir='', threadPool=None):
        self.app = app
        self.tempDir = tempDir
        self.threadPool = threadPool
        self.signals = AstrometrySignals()
        self.process = None
        self.available = {}
        self.result = (False, [])

        if platform.system() == 'Darwin':
            home = os.environ.get('HOME')

            self.binPath = {
                'CloudMakers': '/Applications/Astrometry.app/Contents/MacOS',
            }
            # getting all versions of KStars, of the are multiple versions in app folder
            pattern = re.compile(fnmatch.translate('kstars*.app'), re.IGNORECASE)
            header = '/Applications'
            trailer = '/Contents/MacOS/astrometry/bin'
            for name in sorted(os.listdir('/Applications')):
                if not pattern.match(name):
                    continue
                title = name.strip('.app')
                appPath = f'{header}/{name}{trailer}'
                self.binPath[title] = appPath
            self.indexPath = home + '/Library/Application Support/Astrometry'

        elif platform.system() == 'Linux':
            self.binPath = {
                'astrometry-glob': '/usr/bin',
                'astrometry-local': '/usr/local/astrometry/bin',
            }
            self.indexPath = '/usr/share/astrometry'

        else:
            self.binPath = {}
            self.indexPath = ''

        self.checkAvailability()

    def checkAvailability(self):
        """
        checkAvailability searches for the existence of the core runtime modules from
        astrometry.net namely image2xy and solve-field

        :return: True if local solve and components is available
        """

        self.available = {}
        for app, path in self.binPath.items():
            suc = True
            binPathSolveField = path + '/solve-field'
            binPathImage2xy = path + '/image2xy'

            # checking binaries
            if not os.path.isfile(binPathSolveField):
                self.logger.info(f'{binPathSolveField} not found')
                suc = False
            if not os.path.isfile(binPathImage2xy):
                self.logger.info(f'{binPathImage2xy} not found')
                suc = False

            # checking indexes
            if not glob.glob(self.indexPath + '/index-4*.fits'):
                self.logger.info('no index files found')
                suc = False
            if suc:
                self.available[app] = path
                self.logger.info(f'binary and index files available for {app}')

        return True

    def readFitsData(self, fitsPath):
        """
        readFitsData reads the fits file with the image and tries to get some key
        fields out of the header for preparing the solver.

        :param fitsPath: fits file with image data
        :return: raHint, decHint, scaleHint
        """

        with fits.open(fitsPath) as fitsHDU:
            fitsHeader = fitsHDU[0].header

            # todo: there might be the necessity to read more alternative header info
            # todo: the actual definition is OK for EKOS

            scaleHint = float(fitsHeader.get('SCALE', 0))
            raHint = transform.convertToAngle(fitsHeader.get('RA', 0), isHours=True)
            decHint = transform.convertToAngle(fitsHeader.get('DEC', 0), isHours=False)

        self.logger.debug(f'RA: {raHint}, DEC: {decHint}, Scale: {scaleHint}')

        return raHint, decHint, scaleHint

    @staticmethod
    def getWCSHeader(wcsHDU=''):
        """
        getWCSHeader returns the header part of a fits HDU

        :param wcsHDU: fits file with wcs data
        :return: wcsHeader
        """
        wcsHeader = wcsHDU[0].header
        return wcsHeader

    @staticmethod
    def calcAngleScaleFromWCS(wcsHeader=None):
        """
        calcAngleScaleFromWCS as the name says. important is to use the numpy arctan2
        function, because it handles the zero points and extend the calculation back
        to the full range from -pi to pi

        :return: angle in degrees and scale in arc second per pixel (app) and status if
                 image is flipped
        """

        CD11 = wcsHeader.get('CD1_1', 0)
        CD12 = wcsHeader.get('CD1_2', 0)
        CD21 = wcsHeader.get('CD2_1', 0)
        CD22 = wcsHeader.get('CD2_2', 0)

        flipped = (CD11 * CD22 - CD12 * CD21) < 0

        angleRad = np.arctan2(CD12, CD11)
        angle = np.degrees(angleRad)
        scale = CD11 / np.cos(angleRad) * 3600

        return angle, scale, flipped

    def getSolutionFromWCS(self, fitsHeader=None, wcsHeader=None, updateFits=False):
        """
        getSolutionFromWCS reads the wcs fits file and uses the data in the header
        containing the wcs data and returns the basic data needed.
        in addition it embeds it to the given fits file with image. it removes all
        entries starting with some keywords given in selection. we starting with
        HISTORY

        :param fitsHeader:
        :param wcsHeader:
        :param updateFits:
        :return: ra in hours, dec in degrees, angle in degrees, scale in arcsec/pixel
                 error in arcsec and flag if image is flipped
        """

        raJ2000 = transform.convertToAngle(wcsHeader.get('CRVAL1'), isHours=True)
        decJ2000 = transform.convertToAngle(wcsHeader.get('CRVAL2'), isHours=False)

        if self.app.mainW.ui.enableNoise.isChecked():
            raJ2000 = Angle(hours=raJ2000.hours + np.random.randn() / 10)
            decJ2000 = Angle(degrees=decJ2000.degrees + np.random.randn() / 10)

        angle, scale, flipped = self.calcAngleScaleFromWCS(wcsHeader=wcsHeader)

        raMount = transform.convertToAngle(fitsHeader.get('RA'), isHours=True)
        decMount = transform.convertToAngle(fitsHeader.get('DEC'), isHours=False)

        # todo: it would be nice, if adding, subtracting of angels are part of skyfield
        deltaRA = raJ2000._degrees - raMount._degrees
        deltaDEC = decJ2000.degrees - decMount.degrees
        error = np.sqrt(np.square(deltaRA) + np.square(deltaDEC))
        # would like to have the error RMS in arcsec
        error *= 3600

        solve = Solve(raJ2000=raJ2000,
                      decJ2000=decJ2000,
                      angle=angle,
                      scale=scale,
                      error=error,
                      flipped=flipped,
                      path='')

        if not updateFits:
            return solve, fitsHeader

        remove = ['COMMENT', 'HISTORY']
        fitsHeader.update({k: wcsHeader[k] for k in wcsHeader if k not in remove})

        fitsHeader['RA'] = solve.raJ2000._degrees
        fitsHeader['OBJCTRA'] = transform.convertToHMS(solve.raJ2000)
        fitsHeader['DEC'] = solve.decJ2000.degrees
        fitsHeader['OBJCTDEC'] = transform.convertToDMS(solve.decJ2000)
        fitsHeader['SCALE'] = solve.scale
        fitsHeader['PIXSCALE'] = solve.scale
        fitsHeader['ANGLE'] = solve.angle
        fitsHeader['FLIPPED'] = solve.flipped
        # kee the old values ra, dec as well
        fitsHeader['RA_OLD'] = raMount._degrees
        fitsHeader['DEC_OLD'] = decMount.degrees

        return solve, fitsHeader

    def runImage2xy(self, binPath='', xyPath='', fitsPath='', timeout=30):
        """
        runImage2xy extracts a list of stars out of the fits image. there is a timeout of
        3 seconds set to get the process finished

        :param binPath:   full path to image2xy executable
        :param xyPath:  full path to star file
        :param fitsPath:  full path to fits file
        :param timeout:
        :return: success
        """

        runnable = [binPath,
                    '-O',
                    '-o',
                    xyPath,
                    fitsPath]

        timeStart = time.time()
        try:
            self.process = subprocess.Popen(args=runnable,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE,
                                            )
            stdout, stderr = self.process.communicate(timeout=timeout)
        except subprocess.TimeoutExpired as e:
            self.logger.debug(e)
            return False
        except Exception as e:
            self.logger.error(f'error: {e} happened')
            return False
        else:
            delta = time.time() - timeStart
            self.logger.debug(f'image2xy took {delta}s return code: '
                              + str(self.process.returncode)
                              + ' stderr: '
                              + stderr.decode().replace('\n', ' ')
                              + ' stdout: '
                              + stdout.decode().replace('\n', ' ')
                              )

        success = (self.process.returncode == 0)
        return success

    def runSolveField(self, binPath='', configPath='', xyPath='', options='', timeout=30):
        """
        runSolveField solves finally the xy star list and writes the WCS data in a fits
        file format

        :param binPath:   full path to image2xy executable
        :param configPath: full path to astrometry.cfg file
        :param xyPath:  full path to star file
        :param options: additional solver options e.g. ra and dec hint
        :param timeout:
        :return: success
        """

        runnable = [binPath,
                    '--overwrite',
                    '--no-remove-lines',
                    '--no-plots',
                    '--no-verify-uniformize',
                    '--uniformize', '0',
                    '--sort-column', 'FLUX',
                    '--scale-units', 'app',
                    '--crpix-center',
                    '--cpulimit', str(timeout),
                    '--config',
                    configPath,
                    xyPath,
                    ]

        runnable += options

        timeStart = time.time()
        try:
            self.process = subprocess.Popen(args=runnable,
                                            stdout=subprocess.PIPE,
                                            stderr=subprocess.PIPE
                                            )
            stdout, stderr = self.process.communicate(timeout=timeout)
        except subprocess.TimeoutExpired as e:
            self.logger.debug(e)
            return False
        except Exception as e:
            self.logger.error(f'error: {e} happened')
            return False
        else:
            delta = time.time() - timeStart
            self.logger.debug(f'solve-field took {delta}s return code: '
                              + str(self.process.returncode)
                              + ' stderr: '
                              + stderr.decode().replace('\n', ' ')
                              + ' stdout: '
                              + stdout.decode().replace('\n', ' ')
                              )

        success = (self.process.returncode == 0)

        return success

    def abort(self):
        """

        :return: success
        """

        if self.process:
            self.process.kill()
            return True
        else:
            return False

    def solve(self, app='', fitsPath='', raHint=None, decHint=None, scaleHint=None,
              radius=2, timeout=30, updateFits=False):
        """
        Solve uses the astrometry.net solver capabilities. The intention is to use an
        offline solving capability, so we need a installed instance. As we go multi
        platform and we need to focus on MW function, we use the astrometry.net package
        which is distributed with KStars / EKOS. Many thanks to them providing such a
        nice package.
        As we go using astrometry.net we focus on the minimum feature set possible to
        omit many of the installation and wrapping work to be done. So we only support
        solving of FITS files, use no python environment for astrometry.net parts (as we
        could access these via MW directly)

        The base outside ideas of implementation come from astrometry.net itself and the
        astrometry implementation from cloudmakers.eu (another nice package for MAC Astro
        software)

        :param app: which astrometry implementation to choose
        :param fitsPath:  full path to fits file
        :param raHint:  ra dest to look for solve in J2000
        :param decHint:  dec dest to look for solve in J2000
        :param scaleHint:  scale to look for solve in J2000
        :param radius:  search radius around target coordinates
        :param timeout: time after the subprocess will be killed.
        :param updateFits:  if true update Fits image file with wcsHeader data

        :return: success
        """

        self.process = None
        self.result = Solution(success=False,
                               solve=[])

        if not os.path.isfile(fitsPath):
            return False
        if app not in self.binPath:
            return False

        xyPath = self.tempDir + '/temp.xy'
        configPath = self.tempDir + '/astrometry.cfg'
        solvedPath = self.tempDir + '/temp.solved'
        wcsPath = self.tempDir + '/temp.wcs'
        binPathImage2xy = self.binPath[app] + '/image2xy'
        binPathSolveField = self.binPath[app] + '/solve-field'

        cfgFile = self.tempDir + '/astrometry.cfg'
        with open(cfgFile, 'w+') as outFile:
            outFile.write('cpulimit 300\n')
            outFile.write(f'add_path {self.indexPath}\n')
            outFile.write('autoindex\n')

        suc = self.runImage2xy(binPath=binPathImage2xy,
                               xyPath=xyPath,
                               fitsPath=fitsPath,
                               timeout=timeout,
                               )
        if not suc:
            self.logger.error(f'image2xy error in [{fitsPath}]')
            return False

        raFITS, decFITS, scaleFITS = self.readFitsData(fitsPath=fitsPath)

        # if parameters are passed, they have priority
        if raHint is None:
            raHint = raFITS
        if decHint is None:
            decHint = decFITS
        if scaleHint is None:
            scaleHint = scaleFITS

        searchRatio = 1.1
        ra = transform.convertToHMS(raHint)
        dec = transform.convertToDMS(decHint)
        scaleLow = scaleHint / searchRatio
        scaleHigh = scaleHint * searchRatio
        options = ['--scale-low',
                   f'{scaleLow}',
                   '--scale-high',
                   f'{scaleHigh}',
                   '--ra',
                   f'{ra}',
                   '--dec',
                   f'{dec}',
                   '--radius',
                   f'{radius:1.1f}',
                   ]

        # split between ekos and cloudmakers as cloudmakers use an older version of
        # solve-field, which need the option '--no-fits2fits', whereas the actual
        # version used in KStars throws an error using this option.
        if app == 'CloudMakers':
            options.append('--no-fits2fits')

        suc = self.runSolveField(binPath=binPathSolveField,
                                 configPath=configPath,
                                 xyPath=xyPath,
                                 options=options,
                                 timeout=timeout,
                                 )
        if not suc:
            self.logger.error(f'solve-field error in [{fitsPath}]')
            return False
        if not (os.path.isfile(solvedPath) and os.path.isfile(wcsPath)):
            self.logger.error(f'solve files for [{fitsPath}] missing')
            return False

        with fits.open(wcsPath) as wcsHDU:
            wcsHeader = self.getWCSHeader(wcsHDU=wcsHDU)

        with fits.open(fitsPath, mode='update') as fitsHDU:
            solve, header = self.getSolutionFromWCS(wcsHeader=wcsHeader,
                                                    fitsHeader=fitsHDU[0].header,
                                                    updateFits=updateFits)
            fitsHDU[0].header = header

        solve = Solve(raJ2000=solve.raJ2000,
                      decJ2000=solve.decJ2000,
                      angle=solve.angle,
                      scale=solve.scale,
                      error=solve.error,
                      flipped=solve.flipped,
                      path=fitsPath)
        self.result = Solution(success=True,
                               solve=solve)
        return True

    def solveClear(self):
        """
        the cyclic or long lasting tasks for solving the image should not run
        twice for the same data at the same time. so there is a mutex to prevent this
        behaviour.

        :return: true for test purpose
        """

        self.signals.done.emit(self.result)
        self.signals.message.emit('')

        return True

    def solveThreading(self, app='', fitsPath='', raHint=None, decHint=None, scaleHint=None,
                       radius=2, timeout=30, updateFits=False):
        """
        solveThreading is the wrapper for doing the solve process in a threadpool
        environment of Qt. Otherwise the HMI would be stuck all the time during solving.
        it is done with an securing mutex to avoid starting solving twice. to solveClear
        is the partner of solve Threading

        :param app: which astrometry implementation to choose
        :param fitsPath: full path to the fits image file to be solved
        :param raHint:  ra dest to look for solve in J2000
        :param decHint:  dec dest to look for solve in J2000
        :param scaleHint:  scale to look for solve in J2000
        :param radius:  search radius around target coordinates
        :param timeout: as said
        :param updateFits: flag, if the results should be written to the original file
        :return: success
        """

        if not os.path.isfile(fitsPath):
            self.signals.done.emit(self.result)
            return False
        if not self.checkAvailability():
            self.signals.done.emit(self.result)
            return False

        self.signals.message.emit('solving')
        worker = tpool.Worker(self.solve,
                              app=app,
                              fitsPath=fitsPath,
                              raHint=raHint,
                              decHint=decHint,
                              scaleHint=scaleHint,
                              radius=radius,
                              timeout=timeout,
                              updateFits=updateFits,
                              )
        worker.signals.finished.connect(self.solveClear)
        self.threadPool.start(worker)

        return True
